//--------------------------------------------------------------------------------------------------
#include <iostream>
#include <iomanip>
#include <cstdlib>
#include <cmath>
#include <ctime>
#include <cstdio>
#include <fstream>
#include "TLorentzVector.h"
#include "TFile.h"
#include "TTree.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TPostScript.h"
#include "TLegend.h"
#include "TGraph.h"
#include "TPaveText.h"
#include "TColor.h"
#include "hist_init.h"
#define P7W18 setprecision(7) << setw(18)
#define W3 setw(3)
#define W5 setw(5)
#define W10 setw(10)
using namespace std;
//--------------------------------------------------------------------------------------------------
void get_vectors(double x[], TLorentzVector p[])
{
    // 1 proton (q-bar):
    p[0].SetPxPyPzE(0.0, 0.0, -14000.0, 14000.0);
    // 2 proton (q):
    p[1].SetPxPyPzE(0.0, 0.0,  14000.0, 14000.0);
    // 3 lepton:
    p[2].SetPxPyPzE(x[1], x[2], x[3], x[0]);
    // 4 anti-lepton:
    p[3].SetPxPyPzE(x[5], x[6], x[7], x[4]);
    // 5 photon:
    p[4].SetPxPyPzE(x[9], x[10], x[11], x[8]);
}
//--------------------------------------------------------------------------------------------------
int main ()
{
    gStyle->SetCanvasColor(kWhite);
    gStyle->SetHistFillColor(kWhite);
    gStyle->SetHistLineWidth(1);
    gStyle->SetHistLineStyle(1);
    gStyle->SetPaperSize(20, 28);
    gStyle->SetPalette(1);
    gStyle->SetOptStat(0);
    gStyle->SetLabelFont(42,"X");
    gStyle->SetLabelOffset(0.005,"X");
    gStyle->SetTitleFont(42,"X");
    gStyle->SetTitleSize(0.046,"X");
    gStyle->SetLabelFont(42,"Y");
    gStyle->SetLabelOffset(0.005,"Y");
    gStyle->SetTitleFont(42,"Y");
    gStyle->SetTitleSize(0.046,"Y");
    gStyle->SetTextFont(42);
    TLorentzVector p_tot[5], p_brn[5];

    int iErr = 1;   // 1/0 (histograms with/without errors)
    double x_tot[13], x_brn[13];

    TFile file_tot("events_tot.root", "read", "Events file");
    if(file_tot.IsZombie()) { cout << "Can't open file <events_tot.root>" << endl; return -1; }
    TTree *tree_tot = (TTree*) file_tot.Get("events_tot");
    if(!tree_tot) { cout << "Can't read events tree" << endl; return -2; }
    tree_tot->SetBranchAddress("v_tot", x_tot);
    int n_events_tot = (int) tree_tot->GetEntries();

    TFile file_brn("events_brn.root", "read", "Events file");
    if(file_brn.IsZombie()) { cout << "Can't open file <events_brn.root>" << endl; return -1; }
    TTree *tree_brn = (TTree*) file_brn.Get("events_brn");
    if(!tree_brn) { cout << "Can't read events tree" << endl; return -2; }
    tree_brn->SetBranchAddress("v_brn", x_brn);
    int n_events_brn = (int) tree_brn->GetEntries();
//--------------------------------------------------------------------------------------------------
    double pi = 3.141592653589793;
//--------------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------------
    gStyle->SetHistLineColor(kBlue);
    TH1D h_pt_4_tot
    ("pt_4_tot",
     "Distribution of #mu^{+} transverse momentum;P_{T}(#mu^{+}) [GeV];d#sigma/dP_{T}(#mu^{+}) [pb/GeV]",
     n_bins_pt_4,
     pt_4_left,
     pt_4_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_minv_34_tot
    ("minv_34_tot",
     "Distribution of invariant mass of #mu^{+}#mu^{-} pair;M(#mu^{+}#mu^{-}) [GeV];d#sigma/dM(#mu^{+}#mu^{-}) [pb/GeV]",
     n_bins_minv_34,
     minv_34_left,
     minv_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_minv_34f_tot
    ("minv_34f_tot",
     "",
     n_bins_minv_34,
     minv_34_left,
     minv_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_minv_34b_tot
    ("minv_34b_tot",
     "",
     n_bins_minv_34,
     minv_34_left,
     minv_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_afb1_34_tot
    ("afb1_34_tot",
     "",
     n_bins_minv_34,
     minv_34_left,
     minv_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_afb2_34_tot
    ("afb2_34_tot",
     "",
     n_bins_minv_34,
     minv_34_left,
     minv_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_afb_34_tot
    ("afb_34_tot",
     "Forward-backward assymetry;M(#mu^{+}#mu^{-}) [GeV];A_{FB}",
     n_bins_minv_34,
     minv_34_left,
     minv_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_minv_hm_34_tot
    ("minv_hm_34_tot",
     "Distribution of invariant mass of #mu^{+}#mu^{-} pair;M(#mu^{+}#mu^{-}) [GeV];d#sigma/dM(#mu^{+}#mu^{-}) [pb/GeV]",
     n_bins_minv_hm_34,
     minv_hm_34_left,
     minv_hm_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_minv_hm_34f_tot
    ("minv_hm_34f_tot",
     "",
     n_bins_minv_hm_34,
     minv_hm_34_left,
     minv_hm_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_minv_hm_34b_tot
    ("minv_hm_34b_tot",
     "",
     n_bins_minv_hm_34,
     minv_hm_34_left,
     minv_hm_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_afb1_hm_34_tot
    ("afb1_hm_34_tot",
     "",
     n_bins_minv_hm_34,
     minv_hm_34_left,
     minv_hm_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_afb2_hm_34_tot
    ("afb2_hm_34_tot",
     "",
     n_bins_minv_hm_34,
     minv_hm_34_left,
     minv_hm_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_afb_hm_34_tot
    ("afb_hm_34_tot",
     "Forward-backward assymetry;M(#mu^{+}#mu^{-}) [GeV];A_{FB}",
     n_bins_minv_hm_34,
     minv_hm_34_left,
     minv_hm_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_pr_4_tot
    ("pr_4_tot",
     "Distribution of #mu^{+} pseudo-rapidity;|#eta(#mu^{+})|;d#sigma/d|#eta(#mu^{+})| [pb]",
     n_bins_pr_4,
     pr_4_left,
     pr_4_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_ra_34_tot
    ("ra_34_tot",
     "Distribution of rapidity of #mu^{+}#mu^{-} pair;|y(#mu^{+}#mu^{-})|;d#sigma/d|y(#mu^{+}#mu^{-})| [pb]",
     n_bins_ra_34,
     ra_34_left,
     ra_34_right);
//--------------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------------
    gStyle->SetHistLineColor(kRed);
    TH1D h_pt_4_brn
    ("pt_4_brn",
     "Distribution of #mu^{+} transverse momentum;P_{T}(#mu^{+}) [GeV];d#sigma/dP_{T}(#mu^{+}) [pb/GeV]",
     n_bins_pt_4,
     pt_4_left,
     pt_4_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_minv_34_brn
    ("minv_34_brn",
     "Distribution of invariant mass of #mu^{+}#mu^{-} pair;M(#mu^{+}#mu^{-}) [GeV];d#sigma/dM(#mu^{+}#mu^{-}) [pb/GeV]",
     n_bins_minv_34,
     minv_34_left,
     minv_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_minv_34f_brn
    ("minv_34f_brn",
     "",
     n_bins_minv_34,
     minv_34_left,
     minv_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_minv_34b_brn
    ("minv_34b_brn",
     "",
     n_bins_minv_34,
     minv_34_left,
     minv_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_afb1_34_brn
    ("afb1_34_brn",
     "",
     n_bins_minv_34,
     minv_34_left,
     minv_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_afb2_34_brn
    ("afb2_34_brn",
     "",
     n_bins_minv_34,
     minv_34_left,
     minv_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_afb_34_brn
    ("afb_34_brn",
     "Forward-backward assymetry;M(#mu^{+}#mu^{-}) [GeV];A_{FB}",
     n_bins_minv_34,
     minv_34_left,
     minv_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_minv_hm_34_brn
    ("minv_hm_34_brn",
     "Distribution of invariant mass of #mu^{+}#mu^{-} pair;M(#mu^{+}#mu^{-}) [GeV];d#sigma/dM(#mu^{+}#mu^{-}) [pb/GeV]",
     n_bins_minv_hm_34,
     minv_hm_34_left,
     minv_hm_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_minv_hm_34f_brn
    ("minv_hm_34f_brn",
     "",
     n_bins_minv_hm_34,
     minv_hm_34_left,
     minv_hm_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_minv_hm_34b_brn
    ("minv_hm_34b_brn",
     "",
     n_bins_minv_hm_34,
     minv_hm_34_left,
     minv_hm_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_afb1_hm_34_brn
    ("afb1_hm_34_brn",
     "",
     n_bins_minv_hm_34,
     minv_hm_34_left,
     minv_hm_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_afb2_hm_34_brn
    ("afb2_hm_34_brn",
     "",
     n_bins_minv_hm_34,
     minv_hm_34_left,
     minv_hm_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_afb_hm_34_brn
    ("afb_hm_34_brn",
     "Forward-backward assymetry;M(#mu^{+}#mu^{-}) [GeV];A_{FB}",
     n_bins_minv_hm_34,
     minv_hm_34_left,
     minv_hm_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_pr_4_brn
    ("pr_4_brn",
     "Distribution of #mu^{+} pseudo-rapidity;|#eta(#mu^{+})|;d#sigma/d|#eta(#mu^{+})| [pb]",
     n_bins_pr_4,
     pr_4_left,
     pr_4_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_ra_34_brn
    ("ra_34_brn",
     "Distribution of rapidity of #mu^{+}#mu^{-} pair;|y(#mu^{+}#mu^{-})|;d#sigma/d|y(#mu^{+}#mu^{-})| [pb]",
     n_bins_ra_34,
     ra_34_left,
     ra_34_right);
//--------------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------------
    gStyle->SetHistLineColor(kBlack);
    TH1D h_pt_4_dlt
    ("pt_4_dlt",
     "Distribution of #mu^{+} transverse momentum;P_{T}(#mu^{+}) [GeV];#delta [%]",
     n_bins_pt_4,
     pt_4_left,
     pt_4_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_minv_34_dlt
    ("minv_34_dlt",
     "Distribution of invariant mass of #mu^{+}#mu^{-} pair;M(#mu^{+}#mu^{-}) [GeV];#delta [%]",
     n_bins_minv_34,
     minv_34_left,
     minv_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_minv_34f_dlt
    ("minv_34f_dlt",
     "",
     n_bins_minv_34,
     minv_34_left,
     minv_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_minv_34b_dlt
    ("minv_34b_dlt",
     "",
     n_bins_minv_34,
     minv_34_left,
     minv_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_afb1_34_dlt
    ("afb1_34_dlt",
     "",
     n_bins_minv_34,
     minv_34_left,
     minv_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_afb2_34_dlt
    ("afb2_34_dlt",
     "",
     n_bins_minv_34,
     minv_34_left,
     minv_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_afb_34_dlt
    ("afb_34_dlt",
     "Forward-backward assymetry;M(#mu^{+}#mu^{-}) [GeV];#delta [%]",
     n_bins_minv_34,
     minv_34_left,
     minv_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_minv_hm_34_dlt
    ("minv_hm_34_dlt",
     "Distribution of invariant mass of #mu^{+}#mu^{-} pair;M(#mu^{+}#mu^{-}) [GeV];#delta [%]",
     n_bins_minv_hm_34,
     minv_hm_34_left,
     minv_hm_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_minv_hm_34f_dlt
    ("minv_hm_34f_dlt",
     "",
     n_bins_minv_hm_34,
     minv_hm_34_left,
     minv_hm_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_minv_hm_34b_dlt
    ("minv_hm_34b_dlt",
     "",
     n_bins_minv_hm_34,
     minv_hm_34_left,
     minv_hm_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_afb1_hm_34_dlt
    ("afb1_hm_34_dlt",
     "",
     n_bins_minv_hm_34,
     minv_hm_34_left,
     minv_hm_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_afb2_hm_34_dlt
    ("afb2_hm_34_dlt",
     "",
     n_bins_minv_hm_34,
     minv_hm_34_left,
     minv_hm_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_afb_hm_34_dlt
    ("afb_hm_34_dlt",
     "Forward-backward assymetry;M(#mu^{+}#mu^{-}) [GeV];#delta [%]",
     n_bins_minv_hm_34,
     minv_hm_34_left,
     minv_hm_34_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_pr_4_dlt
    ("pr_4_dlt",
     "Distribution of #mu^{+} pseudo-rapidity;|#eta(#mu^{+})|;#delta [%]",
     n_bins_pr_4,
     pr_4_left,
     pr_4_right);
//--------------------------------------------------------------------------------------------------
    TH1D h_ra_34_dlt
    ("ra_34_dlt",
     "Distribution of rapidity of #mu^{+}#mu^{-} pair;|y(#mu^{+}#mu^{-})|;#delta [%]",
     n_bins_ra_34,
     ra_34_left,
     ra_34_right);
//--------------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------------
    if (iErr == 1) {
        h_pt_4_tot.Sumw2();
        h_minv_34_tot.Sumw2();
        h_minv_34f_tot.Sumw2();
        h_minv_34b_tot.Sumw2();
        h_minv_hm_34_tot.Sumw2();
        h_minv_hm_34f_tot.Sumw2();
        h_minv_hm_34b_tot.Sumw2();
        h_pr_4_tot.Sumw2();
        h_ra_34_tot.Sumw2();

        h_pt_4_brn.Sumw2();
        h_minv_34_brn.Sumw2();
        h_minv_34f_brn.Sumw2();
        h_minv_34b_brn.Sumw2();
        h_minv_hm_34_brn.Sumw2();
        h_minv_hm_34f_brn.Sumw2();
        h_minv_hm_34b_brn.Sumw2();
        h_pr_4_brn.Sumw2();
        h_ra_34_brn.Sumw2();
    }
//--------------------------------------------------------------------------------------------------
    double weight_tot, weight_brn;
    double SigmaBrn, SigmaTot;
    ifstream finBrn ("sigma_brn.dat", ios::in);                                                                                                               
    finBrn >> SigmaBrn;
    ifstream finTot ("sigma_tot.dat", ios::in);                                                                                                               
    finTot >> SigmaTot;
    weight_tot = SigmaTot/(double) n_events_tot;
    weight_brn = SigmaBrn/(double) n_events_brn;
    int ntot = 0;
    int nbrn = 0;
    double costheta;
    for(int i = 0; i < n_events_tot; ++i)
    {

	tree_tot->GetEntry(i);
        get_vectors(x_tot,p_tot);
	if ((p_tot[2].Pt() >= 20.0) && (p_tot[3].Pt() >= 20.0)
            && (abs(p_tot[2].Eta()) <= 2.5) && (abs(p_tot[3].Eta()) <= 2.5)
	    && ((p_tot[2] + p_tot[3]).M() >= 50.0)) {

	    costheta = abs((p_tot[2]+p_tot[3]).Pz())/(p_tot[2]+p_tot[3]).Pz()/(p_tot[2]+p_tot[3]).M()
	    /sqrt((p_tot[2]+p_tot[3]).M()*(p_tot[2]+p_tot[3]).M()+(p_tot[2]+p_tot[3]).Pt()*(p_tot[2]+p_tot[3]).Pt())
	    *((p_tot[2].E()+p_tot[2].Pz())*(p_tot[3].E()-p_tot[3].Pz())-(p_tot[2].E()-p_tot[2].Pz())*(p_tot[3].E()+p_tot[3].Pz()));

	    h_pt_4_tot.Fill   ((p_tot[3]).Pt()                           , x_tot[12]*weight_tot/h_pt_4_tot.GetBinWidth(1));
	    h_minv_34_tot.Fill((p_tot[2] + p_tot[3]).M()                 , x_tot[12]*weight_tot/h_minv_34_tot.GetBinWidth(1));
	    if (costheta >= 0.0) {
	        h_minv_34f_tot.Fill((p_tot[2] + p_tot[3]).M()                 , x_tot[12]*weight_tot/h_minv_34f_tot.GetBinWidth(1));
	    }
	    if (costheta <= 0.0) {
	        h_minv_34b_tot.Fill((p_tot[2] + p_tot[3]).M()                 , x_tot[12]*weight_tot/h_minv_34b_tot.GetBinWidth(1));
	    }
	    h_minv_hm_34_tot.Fill((p_tot[2] + p_tot[3]).M()                 , x_tot[12]*weight_tot/h_minv_hm_34_tot.GetBinWidth(1));
	    if (costheta >= 0.0) {
	        h_minv_hm_34f_tot.Fill((p_tot[2] + p_tot[3]).M()                 , x_tot[12]*weight_tot/h_minv_hm_34f_tot.GetBinWidth(1));
	    }
	    if (costheta <= 0.0) {
	        h_minv_hm_34b_tot.Fill((p_tot[2] + p_tot[3]).M()                 , x_tot[12]*weight_tot/h_minv_hm_34b_tot.GetBinWidth(1));
	    }
	    h_pr_4_tot.Fill   (abs((p_tot[3]).Eta())                          , x_tot[12]*weight_tot/h_pr_4_tot.GetBinWidth(1));
	    h_ra_34_tot.Fill  (abs((p_tot[2]+p_tot[3]).Rapidity())            , x_tot[12]*weight_tot/h_ra_34_tot.GetBinWidth(1));
	    ntot++;
	}
    }
    for(int i = 0; i < n_events_brn; ++i)
    {

	tree_brn->GetEntry(i);
        get_vectors(x_brn,p_brn);
	if ((p_brn[2].Pt() >= 20.0) && (p_brn[3].Pt() >= 20.0)
            && (abs(p_brn[2].Eta()) <= 2.5) && (abs(p_brn[3].Eta()) <= 2.5)
	    && ((p_brn[2] + p_brn[3]).M() >= 50.0)) {

	    costheta = abs((p_brn[2]+p_brn[3]).Pz())/(p_brn[2]+p_brn[3]).Pz()/(p_brn[2]+p_brn[3]).M()
	    /sqrt((p_brn[2]+p_brn[3]).M()*(p_brn[2]+p_brn[3]).M()+(p_brn[2]+p_brn[3]).Pt()*(p_brn[2]+p_brn[3]).Pt())
	    *((p_brn[2].E()+p_brn[2].Pz())*(p_brn[3].E()-p_brn[3].Pz())-(p_brn[2].E()-p_brn[2].Pz())*(p_brn[3].E()+p_brn[3].Pz()));

	    h_pt_4_brn.Fill   ((p_brn[3]).Pt()                           , x_brn[12]*weight_brn/h_pt_4_brn.GetBinWidth(1));
	    h_minv_34_brn.Fill((p_brn[2] + p_brn[3]).M()                 , x_brn[12]*weight_brn/h_minv_34_brn.GetBinWidth(1));
	    if (costheta >= 0.0) {
	        h_minv_34f_brn.Fill((p_brn[2] + p_brn[3]).M()                 , x_brn[12]*weight_brn/h_minv_34f_brn.GetBinWidth(1));
            }
	    if (costheta <= 0.0) {
	        h_minv_34b_brn.Fill((p_brn[2] + p_brn[3]).M()                 , x_brn[12]*weight_brn/h_minv_34b_brn.GetBinWidth(1));
            }
	    h_minv_hm_34_brn.Fill((p_brn[2] + p_brn[3]).M()                 , x_brn[12]*weight_brn/h_minv_hm_34_brn.GetBinWidth(1));
	    if (costheta >= 0.0) {
	        h_minv_hm_34f_brn.Fill((p_brn[2] + p_brn[3]).M()                 , x_brn[12]*weight_brn/h_minv_hm_34f_brn.GetBinWidth(1));
            }
	    if (costheta <= 0.0) {
	        h_minv_hm_34b_brn.Fill((p_brn[2] + p_brn[3]).M()                 , x_brn[12]*weight_brn/h_minv_hm_34b_brn.GetBinWidth(1));
            }
	    h_pr_4_brn.Fill   (abs((p_brn[3]).Eta())                          , x_brn[12]*weight_brn/h_pr_4_brn.GetBinWidth(1));
	    h_ra_34_brn.Fill  (abs((p_brn[2]+p_brn[3]).Rapidity())            , x_brn[12]*weight_brn/h_ra_34_brn.GetBinWidth(1));
	    nbrn++;
        }
    }
    h_afb1_34_tot.Add(&h_minv_34f_tot, &h_minv_34b_tot, 1.0, -1.0);
    h_afb2_34_tot.Add(&h_minv_34f_tot, &h_minv_34b_tot, 1.0, 1.0);
    h_afb_34_tot.Divide(&h_afb1_34_tot, &h_afb2_34_tot, 1.0, 1.0);
    h_afb1_34_brn.Add(&h_minv_34f_brn, &h_minv_34b_brn, 1.0, -1.0);
    h_afb2_34_brn.Add(&h_minv_34f_brn, &h_minv_34b_brn, 1.0, 1.0);
    h_afb_34_brn.Divide(&h_afb1_34_brn, &h_afb2_34_brn, 1.0, 1.0);

    h_afb1_hm_34_tot.Add(&h_minv_hm_34f_tot, &h_minv_hm_34b_tot, 1.0, -1.0);
    h_afb2_hm_34_tot.Add(&h_minv_hm_34f_tot, &h_minv_hm_34b_tot, 1.0, 1.0);
    h_afb_hm_34_tot.Divide(&h_afb1_hm_34_tot, &h_afb2_hm_34_tot, 1.0, 1.0);
    h_afb1_hm_34_brn.Add(&h_minv_hm_34f_brn, &h_minv_hm_34b_brn, 1.0, -1.0);
    h_afb2_hm_34_brn.Add(&h_minv_hm_34f_brn, &h_minv_hm_34b_brn, 1.0, 1.0);
    h_afb_hm_34_brn.Divide(&h_afb1_hm_34_brn, &h_afb2_hm_34_brn, 1.0, 1.0);
    cout << "ntot = " << ntot << endl;
    cout << "nbrn = " << nbrn << endl;
    cout << "sigmatot = " << ntot*weight_tot << endl;
    cout << "sigmabrn = " << nbrn*weight_brn << endl;
//--------------------------------------------------------------------------------------------------
    h_pt_4_dlt.Add   (&h_pt_4_tot   , &h_pt_4_brn   , 100.0, -100.0);
    h_minv_34_dlt.Add(&h_minv_34_tot, &h_minv_34_brn, 100.0, -100.0);
    h_afb_34_dlt.Add(&h_afb_34_tot, &h_afb_34_brn, 100.0, -100.0);
    h_minv_hm_34_dlt.Add(&h_minv_hm_34_tot, &h_minv_hm_34_brn, 100.0, -100.0);
    h_afb_hm_34_dlt.Add(&h_afb_hm_34_tot, &h_afb_hm_34_brn, 100.0, -100.0);
    h_pr_4_dlt.Add   (&h_pr_4_tot   , &h_pr_4_brn   , 100.0, -100.0);
    h_ra_34_dlt.Add  (&h_ra_34_tot   ,&h_ra_34_brn   , 100.0, -100.0);

    h_pt_4_dlt.Divide   (&h_pt_4_dlt   , &h_pt_4_brn   , 1.0, 1.0);
    h_minv_34_dlt.Divide(&h_minv_34_dlt, &h_minv_34_brn, 1.0, 1.0);
    h_afb_34_dlt.Divide(&h_afb_34_dlt, &h_afb_34_brn, 1.0, 1.0);
    h_minv_hm_34_dlt.Divide(&h_minv_hm_34_dlt, &h_minv_hm_34_brn, 1.0, 1.0);
    h_afb_hm_34_dlt.Divide(&h_afb_hm_34_dlt, &h_afb_hm_34_brn, 1.0, 1.0);
    h_pr_4_dlt.Divide   (&h_pr_4_dlt   , &h_pr_4_brn   , 1.0, 1.0);
    h_ra_34_dlt.Divide  (&h_ra_34_dlt  , &h_ra_34_brn  , 1.0, 1.0);
//--------------------------------------------------------------------------------------------------
    ofstream distr_pt_4;
    distr_pt_4.open("distr_pt_4.txt");
    for(int i = 0; i < n_bins_pt_4; ++i) {
       distr_pt_4 << W3 << i+1 << W10 <<  pt_4_left+(i+0.5)*(pt_4_right-pt_4_left)/n_bins_pt_4
       << P7W18 << h_pt_4_brn.GetBinContent(i+1) << P7W18 << h_pt_4_brn.GetBinError(i+1)
       << P7W18 << h_pt_4_tot.GetBinContent(i+1) << P7W18 << h_pt_4_tot.GetBinError(i+1)
       << P7W18 << h_pt_4_dlt.GetBinContent(i+1) << P7W18 << h_pt_4_dlt.GetBinError(i+1)
       << endl;
    }
    distr_pt_4.close();

    ofstream distr_minv_34;
    distr_minv_34.open("distr_minv_34.txt");
    for(int i = 0; i < n_bins_minv_34; ++i) {
       distr_minv_34 << W3 << i+1 << W10 <<  minv_34_left+(i+0.5)*(minv_34_right-minv_34_left)/n_bins_minv_34
       << P7W18 << h_minv_34_brn.GetBinContent(i+1) << P7W18 << h_minv_34_brn.GetBinError(i+1)
       << P7W18 << h_minv_34_tot.GetBinContent(i+1) << P7W18 << h_minv_34_tot.GetBinError(i+1)
       << P7W18 << h_minv_34_dlt.GetBinContent(i+1) << P7W18 << h_minv_34_dlt.GetBinError(i+1)
       << endl;
    }
    distr_minv_34.close();

    ofstream distr_minv_hm_34;
    distr_minv_hm_34.open("distr_minv_hm_34.txt");
    for(int i = 0; i < n_bins_minv_hm_34; ++i) {
       distr_minv_hm_34 << W3 << i+1 << W10 <<  minv_hm_34_left+(i+0.5)*(minv_hm_34_right-minv_hm_34_left)/n_bins_minv_hm_34
       << P7W18 << h_minv_hm_34_brn.GetBinContent(i+1) << P7W18 << h_minv_hm_34_brn.GetBinError(i+1)
       << P7W18 << h_minv_hm_34_tot.GetBinContent(i+1) << P7W18 << h_minv_hm_34_tot.GetBinError(i+1)
       << P7W18 << h_minv_hm_34_dlt.GetBinContent(i+1) << P7W18 << h_minv_hm_34_dlt.GetBinError(i+1)
       << endl;
    }
    distr_minv_hm_34.close();

    ofstream distr_pr_4;
    distr_pr_4.open("distr_pr_4.txt");
    for(int i = 0; i < n_bins_pr_4; ++i) {
       distr_pr_4 << W3 << i+1 << W10 <<  pr_4_left+(i+0.5)*(pr_4_right-pr_4_left)/n_bins_pr_4
       << P7W18 << h_pr_4_brn.GetBinContent(i+1) << P7W18 << h_pr_4_brn.GetBinError(i+1)
       << P7W18 << h_pr_4_tot.GetBinContent(i+1) << P7W18 << h_pr_4_tot.GetBinError(i+1)
       << P7W18 << h_pr_4_dlt.GetBinContent(i+1) << P7W18 << h_pr_4_dlt.GetBinError(i+1)
       << endl;
    }
    distr_pr_4.close();

    ofstream distr_ra_34;
    distr_ra_34.open("distr_ra_34.txt");
    for(int i = 0; i < n_bins_ra_34; ++i) {
       distr_ra_34 << W3 << i+1 << W10 <<  ra_34_left+(i+0.5)*(ra_34_right-ra_34_left)/n_bins_ra_34
       << P7W18 << h_ra_34_brn.GetBinContent(i+1) << P7W18 << h_ra_34_brn.GetBinError(i+1)
       << P7W18 << h_ra_34_tot.GetBinContent(i+1) << P7W18 << h_ra_34_tot.GetBinError(i+1)
       << P7W18 << h_ra_34_dlt.GetBinContent(i+1) << P7W18 << h_ra_34_dlt.GetBinError(i+1)
       << endl;
    }
    distr_ra_34.close();

    ofstream distr_afb_34;
    distr_afb_34.open("distr_afb_34.txt");
    for(int i = 0; i < n_bins_minv_34; ++i) {
       distr_afb_34 << W3 << i+1 << W10 <<  minv_34_left+(i+0.5)*(minv_34_right-minv_34_left)/n_bins_minv_34
       << P7W18 << h_afb_34_brn.GetBinContent(i+1) << P7W18 << h_afb_34_brn.GetBinError(i+1)
       << P7W18 << h_afb_34_tot.GetBinContent(i+1) << P7W18 << h_afb_34_tot.GetBinError(i+1)
       << P7W18 << h_afb_34_dlt.GetBinContent(i+1) << P7W18 << h_afb_34_dlt.GetBinError(i+1)
       << endl;
    }
    distr_afb_34.close();

    ofstream distr_afb_hm_34;
    distr_afb_hm_34.open("distr_afb_hm_34.txt");
    for(int i = 0; i < n_bins_minv_hm_34; ++i) {
       distr_afb_hm_34 << W3 << i+1 << W10 <<  minv_hm_34_left+(i+0.5)*(minv_hm_34_right-minv_hm_34_left)/n_bins_minv_hm_34
       << P7W18 << h_afb_hm_34_brn.GetBinContent(i+1) << P7W18 << h_afb_hm_34_brn.GetBinError(i+1)
       << P7W18 << h_afb_hm_34_tot.GetBinContent(i+1) << P7W18 << h_afb_hm_34_tot.GetBinError(i+1)
       << P7W18 << h_afb_hm_34_dlt.GetBinContent(i+1) << P7W18 << h_afb_hm_34_dlt.GetBinError(i+1)
       << endl;
    }
    distr_afb_hm_34.close();
//--------------------------------------------------------------------------------------------------
    TCanvas canv("canv");
//--------------------------------------------------------------------------------------------------
    TPaveText pt1(0.68, 0.8, 0.88, 0.88, "NDC");
    pt1.SetBorderSize(0);
    pt1.SetTextColor(kRed);
    pt1.AddText("LO");

    TPaveText pt2(0.68, 0.72, 0.88, 0.8, "NDC");
    pt2.SetBorderSize(0);
    pt2.SetTextColor(kBlue);
    pt2.AddText("NLO");
//--------------------------------------------------------------------------------------------------
    h_pt_4_brn.SetMinimum(0);
    h_pt_4_brn.Draw();
    h_pt_4_tot.Draw("SAME");
    pt1.Draw();
    pt2.Draw();
    canv.SetLogy(0);
    canv.Update();
    canv.Print("distr_pt_4_sig.eps");
//--------------------------------------------------------------------------------------------------
    h_minv_34_brn.SetMinimum();
    h_minv_34_brn.Draw();
    h_minv_34_tot.Draw("SAME");
    pt1.Draw();
    pt2.Draw();
    canv.SetLogy(0);
    canv.Update();
    canv.Print("distr_minv_34_sig.eps");
//--------------------------------------------------------------------------------------------------
//    h_afb_34_brn.SetMinimum(-0.2);
//    h_afb_34_brn.SetMaximum(0.1);
//    h_afb_34_brn.Draw();
//    h_afb_34_tot.Draw("SAME");
//    pt1.Draw();
//    pt2.Draw();
//    canv.SetLogy(0);
//    canv.Update();
//    canv.Print("distr_afb_34_sig.eps");
//--------------------------------------------------------------------------------------------------
    h_minv_hm_34_brn.SetMinimum();
    h_minv_hm_34_brn.Draw();
    h_minv_hm_34_tot.Draw("SAME");
    pt1.Draw();
    pt2.Draw();
    canv.SetLogy(0);
    canv.Update();
    canv.Print("distr_minv_hm_34_sig.eps");
//--------------------------------------------------------------------------------------------------
    h_afb_hm_34_tot.SetMinimum(-1.0);
    h_afb_hm_34_tot.SetMaximum(1.0);
    h_afb_hm_34_tot.Draw();
    h_afb_hm_34_brn.Draw("SAME");
    pt1.Draw();
    pt2.Draw();
    canv.SetLogy(0);
    canv.Update();
    canv.Print("distr_afb_hm_34_sig.eps");
//--------------------------------------------------------------------------------------------------
    h_pr_4_tot.SetMinimum(0);
    h_pr_4_tot.Draw();
    h_pr_4_brn.Draw("SAME");
    pt1.Draw();
    pt2.Draw();
    canv.SetLogy(0);
    canv.Update();
    canv.Print("distr_pr_4_sig.eps");
//--------------------------------------------------------------------------------------------------
    h_ra_34_tot.SetMinimum(0);
    h_ra_34_tot.Draw();
    h_ra_34_brn.Draw("SAME");
    pt1.Draw();
    pt2.Draw();
    canv.SetLogy(0);
    canv.Update();
    canv.Print("distr_ra_34_sig.eps");
//--------------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------------
    h_pt_4_dlt.Draw();
    canv.SetLogy(0);
    canv.Update();
    canv.Print("distr_pt_4_dlt.eps");
//--------------------------------------------------------------------------------------------------
    h_minv_34_dlt.Draw();
    canv.SetLogy(0);
    canv.Update();
    canv.Print("distr_minv_34_dlt.eps");
//--------------------------------------------------------------------------------------------------
//    h_afb_34_dlt.Draw();
//    canv.SetLogy(0);
//    canv.Update();
//    canv.Print("distr_afb_34_dlt.eps");
//--------------------------------------------------------------------------------------------------
    h_minv_hm_34_dlt.Draw();
    canv.SetLogy(0);
    canv.Update();
    canv.Print("distr_minv_hm_34_dlt.eps");
//--------------------------------------------------------------------------------------------------
    h_afb_hm_34_dlt.Draw();
    canv.SetLogy(0);
    canv.Update();
    canv.Print("distr_afb_hm_34_dlt.eps");
//--------------------------------------------------------------------------------------------------
    h_pr_4_dlt.Draw();
    canv.SetLogy(0);
    canv.Update();
    canv.Print("distr_pr_4_dlt.eps");
//--------------------------------------------------------------------------------------------------
    h_ra_34_dlt.Draw();
    canv.SetLogy(0);
    canv.Update();
    canv.Print("distr_ra_34_dlt.eps");
//--------------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------------
    file_tot.Close();
    file_brn.Close();
//--------------------------------------------------------------------------------------------------
    return 0;
}
