c
c********************* atlsmear **************************************
c
      subroutine atlsmear(p,sp,lep)
c*********************************************************************
c
c       this subroutne smears the 3-momentum component of the
c       4-vector p.
c
c       this subroutne is suitable for lhc (atlas)
c
c       input:
c      p   : 4-vector to be smeared (real*8)
c      lep : paricle type, 'e' for electron or photon, 'h' for hadron
c    common /iseed1/ iseed1 : integer seed for random number generator
c
c       output:
c       sp  : smeared 4-vector
c
c*********************************************************************
c
      implicit real*8(a-h,o-z)
      character*1 lep
      real*8 scalet
      common/ scalet/ scalet
      common/iseed1/ iseed1
      real*8 p(4),sp(4)
c
      diff = p(4)-p(3)
      sum = p(4)+p(3)
      if (abs(diff).lt.1d-5.or.abs(sum).lt.1d-5) then
         eta = 20.d0
      else
         arg = sum/diff
         if (arg.le.0d0) then
            eta = 20.d0
         elseif (arg.gt.0d0) then
            eta = dabs(.5d0*dlog((p(4)+p(3))/(p(4)-p(3))))
         endif
      endif
      et = dsqrt(p(1)**2+p(2)**2)
      if (lep.eq.'e') then
         if (eta.lt.2.5d0) then
            del = dsqrt(.095d0**2/p(4) + 0.005d0**2 ) * p(4)
         elseif (eta.ge.2.5d0) then
            del = 0.d0
         endif
      endif
      if (lep.eq.'m') then
         if (eta.lt.2.d0) then
            del = dsqrt((5.d-4*et)**2+0.012d0**2)*et
     &           *(1d0+eta**10/7000d0)
         elseif (eta.ge.2.d0) then
            del = 0.d0
         endif
      endif
      if (lep.eq.'n') then
         ret = scalet
         del = 0.45d0*sqrt(ret)
         call rgauss(del,dpx)
         call rgauss(del,dpy)
         sp(1)=p(1)+dpx
         sp(2)=p(2)+dpy
         sp(3)=p(3)
         sp(4)=dsqrt(sp(1)**2+sp(2)**2+sp(3)**2)
         goto 15
      endif
      if (lep.eq.'h') then
         if (eta.lt.3.) then
            del = dsqrt(.5d0**2/p(4) + 0.03d0**2 ) * p(4)
         elseif (eta.ge.3d0.and.eta.lt.4.5d0) then
            del = dsqrt(1d0**2/p(4) + 0.07d0**2 ) * p(4)
         elseif (eta.ge.4.5d0) then
            del = 0.d0
         endif
      endif
 9    call rgauss(del,dp)
      if (p(4).gt.0.d0 .and. dp.gt.-p(4) ) then
         sp(4) = p(4) + dp
         do 10 i=1,3
            sp(i)=p(i)*(1d0+dp/p(4))
 10      continue
      else
         sp(4) = 1d-15 * p(4)
         do 11 j=1,3
            sp(j)=1d-15*p(j)
 11      continue
      endif
 15   return
      end
c
c*********************** cmssmear  **************************************
c
      subroutine cmssmear(p,sp,lep)

c***************************************************************************
c
c       this subroutne smears the 3-momentum component of the 4-vector p.
c
c       this subroutne is suitable for lhc (cms)
c
c       input:
c       p   : 4-vector to be smeared (real*8)
c       lep : paricle type, 'e' for electron or photon, 'h' for hadron
c       common /iseed1/ iseed1 : integer seed for random number generator
c
c       output:
c       sp  : smeared 4-vector
c
c***************************************************************************
*
c
      implicit real*8(a-h,o-z)
      character*1 lep
      common/iseed1/ iseed1
      real*8 p(4),sp(4)
c
      diff = p(4)-p(3)
      sum = p(4)+p(3)
      if (abs(diff).lt.1.e-5.or.abs(sum).lt.1.e-5) then
        eta = 20.
      else
        arg = sum/diff
        if (arg.le.0.) then
          eta = 20.
        elseif (arg.gt.0.) then
          eta = abs(.5*log((p(4)+p(3))/(p(4)-p(3))))
        endif
      endif
      et = sqrt(p(1)**2+p(2)**2)
      if (lep.eq.'e') then
        if (eta.lt.2.6) then
          del = sqrt(.02**2/p(4) + 0.005**2 + 0.15**2/p(4)**2 ) * p(4)
        elseif (eta.ge.2.6) then
          del = 0.
        endif
      endif
      if (lep.eq.'n') then
        del = 0.
        do i=1,2
          sp(i) = p(i)
        end do
        return
      endif
      if (lep.eq.'h') then
        if (eta.lt.2.6) then
          del = sqrt(.52**2/p(4) + 0.015**2 ) * p(4)
        elseif (eta.ge.2.6.and.eta.lt.4.7) then
          del = sqrt(1.25**2/p(4) + 0.05**2 ) * p(4)
        elseif (eta.ge.4.7) then
          del = 0.
        endif
      endif
    9 call rgauss(del,dp)
      if (p(4).gt.0. .and. dp.gt.-p(4) ) then
        sp(4) = p(4) + dp
        do 10 i=1,3
   10   sp(i)=p(i)*(1.+dp/p(4))
      else
        sp(4) = 1d-15 * p(4)
        do 11 j=1,3
   11   sp(j)=1d-15*p(j)
      endif
   15 return
      end
c
c*********************** cdfsmear  
*******************************************
c
      subroutine cdfsmear(p,sp,lep)
c
c***************************************************************************
c
c       this subroutne smears the 3-momentum component of the 4-vector p.
c
c       this subroutine is suitable for tevatron (cdf inspired)
c
c       input:
c       p   : 4-vector to be smeared (real*8)
c       lep : paricle type, 'e' for electron or photon, 'h' for hadron,
c             'n' for missing pt
c       common /iseed1/ iseed1 : integer seed for random number generator
c
c       output:
c       sp  : smeared 4-vector
c
c***************************************************************************
*
c
      implicit real*8(a-h,o-z)
      character*1 lep
      real*8 scalet
      common/ scalet/ scalet
      common/iseed1/ iseed1
      real*8 p(4),sp(4)
c
      diff = p(4)-p(3)
      sum = p(4)+p(3)
      if (abs(diff).lt.1.e-5.or.abs(sum).lt.1.e-5) then
        eta = 20.
      else
        arg = sum/diff
        if (arg.le.0.) then
          eta = 20.
        elseif (arg.gt.0.) then
          eta = abs(.5*log((p(4)+p(3))/(p(4)-p(3))))
        endif
      endif
      et = sqrt(p(1)**2+p(2)**2)
      if (lep.eq.'e') then
        if (eta.lt.1.2) then
          del=sqrt(.135**2/et + 0.01**2)*p(4)
        elseif (eta.ge.1.2.and.eta.lt.2.4) then
          del=sqrt(.28**2/p(4) + 0.01**2)*p(4)
        elseif (eta.ge.2.4.and.eta.lt.4.2) then
          del=sqrt(.25**2/p(4) + 0.01**2)*p(4)
        elseif (eta.ge.4.2) then
          del=0.
        endif
        goto 9
      endif
      if (lep.eq.'h') then
        if (eta.lt.1.3) then
          del=sqrt(.75**2/et + 0.03**2)*p(4)
        elseif (eta.ge.1.3.and.eta.lt.2.4) then
          del=sqrt(.90**2/p(4) + 0.04**2)*p(4)
        elseif (eta.ge.2.4.and.eta.lt.4.2) then
          del=sqrt(1.30**2/p(4) + 0.04**2)*p(4)
        elseif (eta.ge.4.2) then
          del=0.
        endif
        goto 9
      endif
      if (lep.eq.'n') then
        ret = scalet
        del=.47*sqrt(ret)
        call rgauss(del,dpx)
        call rgauss(del,dpy)
        sp(1)=p(1)+dpx
        sp(2)=p(2)+dpy
        sp(3)=p(3)
        sp(4)=p(4)
        goto 15
      endif

    9 continue
c
c     del fixed at % for test.
c
c      del=5.

      call rgauss(del,dp)
      if (p(4).gt.0. .and. dp.gt.-p(4) ) then
        sp(4) = p(4) + dp
        do 10 i=1,3
   10   sp(i)=p(i)*(1.+dp/p(4))
      else
        sp(4) = 1d-10 * p(4)
        do 11 j=1,3
   11   sp(j)=1d-10*p(j)
      endif
   15 return
      end
c
c*********************** d0smear  
*******************************************
c
      subroutine d0smear(p,sp,lep)
c
c***************************************************************************
c
c       this subroutne smears the 3-momentum component of the 4-vector p.
c
c       this subroutine is suitable for tevatron (d0 inspired)
c
c       input:
c       p   : 4-vector to be smeared (real*8)
c       lep : paricle type, 'e' for electron or photon, 'h' for hadron,
c             'n' for missing pt
c       common /iseed1/ iseed1 : integer seed for random number generator
c
c       output:
c       sp  : smeared 4-vector
c
c***************************************************************************
*
c
      implicit real*8(a-h,o-z)
      character*1 lep
      real*8 scalet
      common/ scalet/ scalet
      common/iseed1/ iseed1
      real*8 p(4),sp(4)
c
      diff = p(4)-p(3)
      sum = p(4)+p(3)
      if (abs(diff).lt.1.e-5.or.abs(sum).lt.1.e-5) then
        eta = 20.
      else
        arg = sum/diff
        if (arg.le.0.) then
          eta = 20.
        elseif (arg.gt.0.) then
          eta = abs(.5*log((p(4)+p(3))/(p(4)-p(3))))
        endif
      endif
      et = sqrt(p(1)**2+p(2)**2)
      if (lep.eq.'e') then
        c = 0.006
        if (eta.lt.1.3) then
          s = 0.142
        elseif (eta.ge.1.3.and.eta.lt.4.) then
          s = 0.163
        endif
        an = 0.32
        if (eta.lt.4.) then
          del = sqrt( s**2/p(4) + c**2 + (an/p(4))**2 )*p(4)
        elseif (eta.ge.4.) then
          del = 0.
        endif
        goto 9
      endif
      if (lep.eq.'h') then
        del = sqrt(0.5**2/et + 0.05**2)*p(4)
        goto 9
      endif
      if (lep.eq.'n') then
        ret = scalet
        del = 2.94 + 2.86e-05 * ret**2
        call rgauss(del,dpx)
        call rgauss(del,dpy)
        sp(1)=p(1)+dpx
        sp(2)=p(2)+dpy
        sp(3)=p(3)
        sp(4)=p(4)
        goto 15
      endif
    9 call rgauss(del,dp)
      if (p(4).gt.0. .and. dp.gt.-p(4) ) then
        sp(4) = p(4) + dp
        do 10 i=1,3
   10   sp(i)=p(i)*(1.+dp/p(4))
      else
        sp(4) = 1d-10 * p(4)
        do 11 j=1,3
   11   sp(j)=1d-10*p(j)
      endif
   15 return
      end
c
c***************************************************************************
c
      subroutine rgauss(del,dp)
c-------yields gaussian distribution of random numbers
      implicit real*8(a-h,o-z)
      common/iseed1/ iseed1
      data iseed1 /23147/
      real*8 x(2)
      
      if (del.le.0.) then
        dp = 0.
        return
      endif 

 5    call randa(2,x)
      
      dp=3d0*del*(-1d0+2d0*x(1))

      gauss=exp(-dp**2/2d0/del**2)
      if (x(2)-gauss) 10,10,5
 10   return
      end
c
c*********************** nsmear  ******************************************
c
      subroutine nsmear(p,sp,lep)
c
c***************************************************************************
c
c       this subroutne does not smear
c
c***************************************************************************
*
c
      implicit real*8(a-h,o-z)
      character*1 lep
      real*8 p(4),sp(4)
      do 1 i=1,4
    1 sp(i) = p(i)
      return
      end



      subroutine metsmear(p,sp)
c
c***************************************************************************
c
c       this subroutine smears the pt-momentum component of the 4-vector p. of
c       a neutrino
c     
c       this subroutine is suitable for tevatron (d0) inspired)
c
c       input:
c       p   : 4-vector to be smeared (real*8)
c       sumet: sum of the et in the event
c
c       output:
c       sp  : smeared 4-vector
c
c***************************************************************************
c
      implicit real*8(a-h,o-z)
      real*8 p(4),sp(4)
cac      real*4 x
      real*8 pi
      parameter(pi=3.1415926535987932)

c
      et=sqrt(p(1)**2+p(2)**2)
      del=5.

      call rgauss(del,dp)
c
c     if et=0, then smear in random direction
c
      if(et.eq.0.0)then
 5       call randa(1,x)
         sp(1)=dp*cos(x*pi)
         sp(2)=dp*sin(x*pi)
      else
         do i=1,2
            sp(i)=p(i)*(1.d0+dp/et)
         end do
      end if
      sp(3)=p(3)
      sp(4)=sqrt(sp(3)**2+sp(2)**2+sp(1)**2)
      return
      
      end

      subroutine et4smear(p,sp)
c
c***************************************************************************
c
c       this subroutne smears the pt-momentum component of the 4-vector p. 
c       by 4 gev in a random direction 
c     
c       this subroutne is suitable for tevatron
c
c       input:
c       p   : 4-vector to be smeared (real*8)
c       sumet: sum of the et in the event
c
c       output:
c       sp  : smeared 4-vector
c
c***************************************************************************
c
      implicit real*8(a-h,o-z)
      real*8 p(4),sp(4)
cac      real*4 x
      real*8 pi
      parameter(pi=3.1415926535987932)

      del=4.
      call rgauss(del,dp)
      call randa(1,x)
      sp(1)=p(1)+dp*cos(x*pi)
      sp(2)=p(2)+dp*sin(x*pi)
      sp(3)=p(3)
      sp(4)=sqrt(sp(3)**2+sp(2)**2+sp(1)**2)
      return
      
      end
C
C*********************** D0UPGRSMEAR  *******************************************
C
      subroutine d0upgrsmear(p,sp,lep)
c
c***************************************************************************
c
c       this subroutine smears the 3-momentum component of the 4-vector p.
c
c       this subroutne is suitable for tevatron (upgraded d0)
c
c       input:
c       p   : 4-vector to be smeared (real*8)
c       lep : paricle type, 'e' for electron or photon, 'h' for hadron,
c             'n' for missing pt
c       common /iseed1/ iseed1 : integer seed for random number generator
c
c       output:
c       sp  : smeared 4-vector
c
c     updated 05/11/95
c
c****************************************************************************
c
      implicit real*8(a-h,o-z)
      character*1 lep
      real*8 scalet
      common/ scalet/ scalet
      common/iseed1/ iseed1
      real*8 p(4),sp(4)
c
      if (p(4).ne.p(3)) then
        ratio = (p(4)+p(3))/(p(4)-p(3))
        if (ratio.gt.0.) then
          eta = abs(.5*log((p(4)+p(3))/(p(4)-p(3))))
        else
          eta = 10.
        endif
      else
        eta = 10.
      endif
      et = sqrt(p(1)**2+p(2)**2)
c  electrons
      if (lep.eq.'e') then
        c = 0.003
        if (eta.lt.1.3) then
          s = 0.140
          en = 0.140
        elseif (eta.ge.1.3.and.eta.lt.4.) then
          s = 0.157
          en = 0.290
        endif
        if (eta.lt.4.) then
          del = sqrt( (en/p(4))**2 + s**2/p(4) + c**2 )*p(4)
        elseif (eta.ge.4.) then
          del = 0.
        endif
        goto 9
      endif
c  muons
      if (lep.eq.'m') then
        a = 0.015
        b = 0.0016
        if (eta.lt.1.6) then
          del = sqrt( (b*et)**2 + a**2 )*et
        else
          del = 0.
        endif
        goto 9
      endif
c jets
      if (lep.eq.'h') then
        del = sqrt((2.16/et)**2 + 0.74**2/et + 0.01**2)*et
        goto 9
      endif
c missing et
      if (lep.eq.'n') then
        ret = scalet
        del = 1.08 + 0.019 * ret
        call rgauss(del,dpx)
        call rgauss(del,dpy)
        sp(1)=p(1)+dpx
        sp(2)=p(2)+dpy
        sp(3)=p(3)
        sp(4)=sqrt(sp(1)**2+sp(2)**2+sp(3)**2)
        goto 15
      endif
    9 call rgauss(del,dp)
      if (p(4).gt.0. .and. dp.gt.-p(4) ) then
        sp(4) = p(4) + dp
        do 10 i=1,3
   10   sp(i)=p(i)*(1.+dp/p(4))
      else
        sp(4) = 1d-6 * p(4)
        do 11 j=1,3
   11   sp(j)=1d-6*p(j)
      endif
   15 return
      end
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      subroutine randa (n, ran)
C...Subroutine generates uniformly distributed random
C...numbers x (i), i = 1, n
      implicit none
      integer n
      double precision ran(n), randdd
      integer i
      do i = 1,n
        ran (i) = randdd (n)
      enddo
      return
      end
C-----------------------------------------------------------------------
       double precision function ranf(dummy)
*
*      random number function taken from knuth
*      (seminumerical algorithms).
*      method is x(n)=mod(x(n-55)-x(n-24),1/fmodul)
*      no provision yet for control over the seed number.
*
*      ranf gives one random number between 0 and 1.
*      irn55 generates 55 random numbers between 0 and 1/fmodul.
*      in55  initializes the 55 numbers and warms up the sequence.
*
       implicit double precision (a-h,o-z)
       parameter (fmodul=1.d-09)
       integer ia(55)
       save ia
       data ncall/0/
       data mcall/55/
       if( ncall.eq.0 ) then
           call in55 ( ia,234612957 )
           ncall = 1
       endif
       if ( mcall.eq.0 ) then
           call irn55(ia)
           mcall=55
       endif
       ranf=ia(mcall)*fmodul
       mcall=mcall-1
       end

       subroutine in55(ia,ix)
       parameter (modulo=1000000000)
       integer ia(55)
       ia(55)=ix
       j=ix
       k=1
       do 10 i=1,54
       ii=mod(21*i,55)
       ia(ii)=k
       k=j-k
       if(k.lt.0)k=k+modulo
       j=ia(ii)
 10      continue
       do 20 i=1,10
       call irn55(ia)
 20      continue
       end

       subroutine irn55(ia)
       parameter (modulo=1000000000)
       integer ia(55)
       do 10 i=1,24
       j=ia(i)+ia(i+31)
       if(j.ge.modulo)j=j-modulo
       ia(i)=j
 10      continue
       do 20 i=25,55
       j=ia(i)+ia(i-24)
       if(j.ge.modulo)j=j-modulo
       ia(i)=j
 20      continue
       end
c************************ Random number generator *******
        real*8 function ranini(n)
        implicit real*8 (a-h,o-z)
        real*8 randdd
C INITIALIZE
        dimension r3(127)
        data r1/2.d0/,s/0.d0/,t/1.d0/,rmc/1.d0/,iw/-1/
        save
        if(n.eq.0)n=12345
C MUST HAVE NON-ZERO INITIALISER
        if(iw.gt.0)goto 30
10      iw=iw+1
        t=.5d0*t
        r1=s
        s=s+t
        if((s.gt.r1).and.(s.lt.1.d0))goto 10
        ikt=(iw-1)/12
        ic=iw-12*ikt
        id=2**(13-ic)
        do 20 i=1,ic
20      rmc=.5d0*rmc
        rm=.015625d0*.015625d0
30      i2=127
        ir=mod(iabs(n),8190)+1
40      r1=0.d0
        do 50 i=1,ikt
        ir=mod(17*ir,8191)
50      r1=(r1+float(ir/2))*rm
        ir=mod(17*ir,8191)
        r1=(r1+float(ir/id))*rmc
        r3(i2)=r1
        i2=i2-1
        if(i2.gt.0)goto 40
        RANINI=R1
        RETURN
        ENTRY RANDDD(N)
C NEW ENTRY POINT
        if(i2.eq.0)i2=127
        RANDDD=r1+r3(i2)
        if(RANDDD.ge.1.)RANDDD=(r1-.5d0)+(r3(i2)-.5d0)
        r1=RANDDD
        r3(i2)=RANDDD
        i2=i2-1
        return
        end
