//--------------------------------------------------------------------------------------------------
#include<fstream>
#include<cmath>
#include<iomanip>
using namespace std;
#include "TPSEMAR.h"
#include "TFDISTR.h"
#include "TFOAM_INTEGRAND.h"
#include "TFOAM.h"
#include "TFile.h"
#include "foam_init.h"
#define SP12 setprecision(7) << setw(18)
#define SW18 setw(18)
#define SW3 setw(3)
//--------------------------------------------------------------------------------------------------
extern struct {
    int wgt;
} weight_;
//--------------------------------------------------------------------------------------------------
extern struct {
    double betaf,cosf,sinf,mf,mf1;
} kinematics_;
//--------------------------------------------------------------------------------------------------
extern struct {
    double sqs0;
} ebeam_;
//--------------------------------------------------------------------------------------------------
extern struct {
    double xl[3],xu[3];
} softlimits_;
//--------------------------------------------------------------------------------------------------
extern struct {
    double xl[6],xu[6];
} hardlimits_;
//--------------------------------------------------------------------------------------------------
extern struct {
    double xl[6],xu[6];
} ibbrlimits_;
//--------------------------------------------------------------------------------------------------
extern struct {
    double xl[4],xu[4];
} ibsulimits_;
//--------------------------------------------------------------------------------------------------
void get_vectors_hard(double x[], double p[][4])
{
    double z1,z2,z3,z4,z5,z6;
    double xl[6],xu[6];
    double sqs0,s0;
    double mmo,mmo2;
    double pi;
    double y,ph4,cth4,cth5,wy,x2;
    double x1,s,sqs,spr,sqspr;
    double cph4,sph4,sth4,sth5;
    double beta,gamma,beta1,gamma1,betal;
    double pmi1[4],ppl1[4],pph1[4],pmi2[4],ppl2[4],pph2[4];

    pi = 3.141592653589793;
    mmo  = kinematics_.mf;
    mmo2 = mmo*mmo;
    sqs0 = ebeam_.sqs0;
    s0   = sqs0*sqs0;

    xl[0] = hardlimits_.xl[0];
    xu[0] = hardlimits_.xu[0];
    xl[1] = hardlimits_.xl[1];
    xu[1] = hardlimits_.xu[1];
    xl[2] = hardlimits_.xl[2];
    xu[2] = hardlimits_.xu[2];
    xl[3] = hardlimits_.xl[3];
    xu[3] = hardlimits_.xu[3];
    xl[4] = hardlimits_.xl[4];
    xu[4] = hardlimits_.xu[4];
    xl[5] = hardlimits_.xl[5];
    xu[5] = hardlimits_.xu[5];

    z1 = xl[0] + x[0]*(xu[0] - xl[0]);
    z2 = xl[1] + x[1]*(xu[1] - xl[1]);
    z3 = xl[2] + x[2]*(xu[2] - xl[2]);
    z4 = xl[3] + x[3]*(xu[3] - xl[3]);
    z5 = xl[4] + x[4]*(xu[4] - xl[4]);
    z6 = xl[5] + x[5]*(xu[5] - xl[5]);

    y    = z1;
    ph4  = z2;
    cth4 = z3;
    cth5 = z4;
    wy   = z5;
    x2   = z6;

    x1    = wy/x2;
    s     = s0*x1*x2;
    sqs   = sqrt(s);
    spr   = y*wy*s0;
    sqspr = sqrt(spr);

    cph4 = cos(ph4);
    sph4 = sin(ph4);
    sth4 = sqrt(1.0-cth4*cth4);
    sth5 = sqrt(1.0-cth5*cth5);

    beta   = (x2-x1)/(x1+x2);
    gamma  = 1.0/sqrt(1.0-beta*beta);
    beta1  = (s-spr)/(s+spr);
    gamma1 = 1.0/sqrt(1.0-beta1*beta1);
    betal  = sqrt(1.0-4.0*mmo2/spr);

    pmi1[1] = -sqspr/2.0*betal*sth4*cph4;
    pmi1[2] = -sqspr/2.0*betal*sth4*sph4;
    pmi1[3] = (-sqspr/2.0*betal*cth4-beta1*sqspr/2.0)*gamma1;
    pmi1[0] = (sqspr/2.0+beta1*sqspr/2.0*betal*cth4)*gamma1;

    ppl1[1] = sqspr/2.0*betal*sth4*cph4;
    ppl1[2] = sqspr/2.0*betal*sth4*sph4;
    ppl1[3] = (sqspr/2.0*betal*cth4-beta1*sqspr/2.0)*gamma1;
    ppl1[0] = (sqspr/2.0-beta1*sqspr/2.0*betal*cth4)*gamma1;

    pph1[1] = 0.0;
    pph1[2] = 0.0;
    pph1[3] = (s-spr)/2.0/sqs;
    pph1[0] = (s-spr)/2.0/sqs;

    pmi2[1] = pmi1[1];
    pmi2[2] = pmi1[2]*cth5+pmi1[3]*sth5;
    pmi2[3] = -pmi1[2]*sth5+pmi1[3]*cth5;
    pmi2[0] = pmi1[0];

    ppl2[1] = ppl1[1];
    ppl2[2] = ppl1[2]*cth5+ppl1[3]*sth5;
    ppl2[3] = -ppl1[2]*sth5+ppl1[3]*cth5;;
    ppl2[0] = ppl1[0];

    pph2[1] = pph1[1];
    pph2[2] = pph1[2]*cth5+pph1[3]*sth5;
    pph2[3] = -pph1[2]*sth5+pph1[3]*cth5;
    pph2[0] = pph1[0];

// lepton:
    p[0][1] = pmi2[1];
    p[0][2] = pmi2[2];
    p[0][3] = (pmi2[3]+beta*pmi2[0])*gamma; //!!!!!!!!!
    p[0][0] = (beta*pmi2[3]+pmi2[0])*gamma;
// anti-lepton:
    p[1][1] = ppl2[1];
    p[1][2] = ppl2[2];
    p[1][3] = (ppl2[3]+beta*ppl2[0])*gamma; //!!!!!!!!!
    p[1][0] = (beta*ppl2[3]+ppl2[0])*gamma;
// photon:
    p[2][1] = pph2[1];
    p[2][2] = pph2[2];
    p[2][3] = (pph2[3]+beta*pph2[0])*gamma; //!!!!!!!!!
    p[2][0] = (beta*pph2[3]+pph2[0])*gamma;
}
//--------------------------------------------------------------------------------------------------
void get_vectors_soft(double x[], double p[][4])
{
    double z1,z2,z3;
    double xl[3],xu[3];
    double sqs0,s0;
    double x1,x2,sqs,s,cti;

    sqs0 = ebeam_.sqs0;
    s0   = sqs0*sqs0;

    xl[0] = softlimits_.xl[0];
    xu[0] = softlimits_.xu[0];
    xl[1] = softlimits_.xl[1];
    xu[1] = softlimits_.xu[1];
    xl[2] = softlimits_.xl[2];
    xu[2] = softlimits_.xu[2];

    z1 = xl[0] + x[0]*(xu[0] - xl[0]);
    z2 = xl[1] + x[1]*(xu[1] - xl[1]);
    z3 = xl[2] + x[2]*(xu[2] - xl[2]);

    x1  = z2/z3;
    x2  = z3;
    s   = s0*x1*x2;
    sqs = sqrt(s);
    cti = 1.0 - 2.0*(1.0 - z1)*x1/(x1 + x2 - z1*(x1 - x2));
// lepton:
    p[0][1] = 0.0;
    p[0][2] = -sqs/2.0*sqrt(1.0 - cti*cti);
    p[0][3] = sqs0/2.0*(x1 - x2 - 2.0*x1*x2/(x1 + x2 - z1*(x1 - x2))*z1);
    p[0][0] = sqs0/2.0*(x1 + x2 - 2.0*x1*x2/(x1 + x2 - z1*(x1 - x2)));
// anti-lepton:
    p[1][1] = 0.0;
    p[1][2] = sqs/2.0*sqrt(1.0 - cti*cti);
    p[1][3] = sqs0*x1*x2/(x1 + x2 - z1*(x1 - x2))*z1;
    p[1][0] = sqs0*x1*x2/(x1 + x2 - z1*(x1 - x2));
// photon:
    p[2][1] = 0.0;
    p[2][2] = 0.0;
    p[2][3] = 0.0;
    p[2][0] = 0.0;
}
//--------------------------------------------------------------------------------------------------
void get_vectors_ibbr(double x[], double p[][4])
{
    double z1,z2,z3,z4,z5,z6;
    double xl[6],xu[6];
    double sqs0,s0;
    double mmo,mmo2;
    double pi;
    double y,ph4,cth4,cth2,wy,x2;
    double x1,s,sqs,spr,sqspr;
    double cph4,sph4,sth4,sth2;
    double beta,gamma,beta1,gamma1,betal;
    double pmi1[4],ppl1[4],pmi2[4],ppl2[4];

    pi = 3.141592653589793;
    mmo  = kinematics_.mf;
    mmo2 = mmo*mmo;
    sqs0 = ebeam_.sqs0;
    s0   = sqs0*sqs0;

    xl[0] = ibbrlimits_.xl[0];
    xu[0] = ibbrlimits_.xu[0];
    xl[1] = ibbrlimits_.xl[1];
    xu[1] = ibbrlimits_.xu[1];
    xl[2] = ibbrlimits_.xl[2];
    xu[2] = ibbrlimits_.xu[2];
    xl[3] = ibbrlimits_.xl[3];
    xu[3] = ibbrlimits_.xu[3];
    xl[4] = ibbrlimits_.xl[4];
    xu[4] = ibbrlimits_.xu[4];
    xl[5] = ibbrlimits_.xl[5];
    xu[5] = ibbrlimits_.xu[5];

    z1 = xl[0] + x[0]*(xu[0] - xl[0]);
    z2 = xl[1] + x[1]*(xu[1] - xl[1]);
    z3 = xl[2] + x[2]*(xu[2] - xl[2]);
    z4 = xl[3] + x[3]*(xu[3] - xl[3]);
    z5 = xl[4] + x[4]*(xu[4] - xl[4]);
    z6 = xl[5] + x[5]*(xu[5] - xl[5]);

    y    = z1;
    cth2 = z2;
    ph4  = z3;
    cth4 = z4;
    wy   = z5;
    x2   = z6;

    x1    = wy/x2;
    s     = s0*x1*x2;
    sqs   = sqrt(s);
    spr   = y*wy*s0;
    sqspr = sqrt(spr);

    cph4 = cos(ph4);
    sph4 = sin(ph4);
    sth4 = sqrt(1.0-cth4*cth4);
    sth2 = sqrt(1.0-cth2*cth2);

    beta   = (x1-x2)/(x1+x2);
    gamma  = 1.0/sqrt(1.0-beta*beta);
    beta1  = (s-spr)/(s+spr);
    gamma1 = 1.0/sqrt(1.0-beta1*beta1);
    betal  = sqrt(1.0-4.0*mmo2/spr);

    pmi1[1] = -sqspr/2.0*betal*sth4*cph4;
    pmi1[2] = -sqspr/2.0*betal*sth4*sph4;
    pmi1[3] = (-sqspr/2.0*betal*cth4+beta1*sqspr/2.0)*gamma1;
    pmi1[0] = (sqspr/2.0-beta1*sqspr/2.0*betal*cth4)*gamma1;

    ppl1[1] = sqspr/2.0*betal*sth4*cph4;
    ppl1[2] = sqspr/2.0*betal*sth4*sph4;
    ppl1[3] = (sqspr/2.0*betal*cth4+beta1*sqspr/2.0)*gamma1;
    ppl1[0] = (sqspr/2.0+beta1*sqspr/2.0*betal*cth4)*gamma1;

    pmi2[1] = pmi1[1]*cth2-pmi1[3]*sth2;
    pmi2[2] = pmi1[2];
    pmi2[3] = pmi1[1]*sth2+pmi1[3]*cth2;
    pmi2[0] = pmi1[0];

    ppl2[1] = ppl1[1]*cth2-ppl1[3]*sth2;
    ppl2[2] = ppl1[2];
    ppl2[3] = ppl1[1]*sth2+ppl1[3]*cth2;
    ppl2[0] = ppl1[0];

// lepton:
    p[0][1] = pmi2[1];
    p[0][2] = pmi2[2];
    p[0][3] = (pmi2[3]+beta*pmi2[0])*gamma;
    p[0][0] = (beta*pmi2[3]+pmi2[0])*gamma;
// anti-lepton:
    p[1][1] = ppl2[1];
    p[1][2] = ppl2[2];
    p[1][3] = (ppl2[3]+beta*ppl2[0])*gamma;
    p[1][0] = (beta*ppl2[3]+ppl2[0])*gamma;
// photon:
    p[2][1] = 0.0;
    p[2][2] = 0.0;
    p[2][3] = 0.0;
    p[2][0] = 0.0;
}
//--------------------------------------------------------------------------------------------------
void get_vectors_ibsu(double x[], double p[][4])
{
    double z1,z2,z3,z4;
    double xl[4],xu[4];
    double sqs0,s0;
    double wy,cti;
    double x1,x2,x3,c,s,sqs,aaa;

    sqs0 = ebeam_.sqs0;
    s0   = sqs0*sqs0;

    xl[0] = ibsulimits_.xl[0];
    xu[0] = ibsulimits_.xu[0];
    xl[1] = ibsulimits_.xl[1];
    xu[1] = ibsulimits_.xu[1];
    xl[2] = ibsulimits_.xl[2];
    xu[2] = ibsulimits_.xu[2];
    xl[3] = ibsulimits_.xl[3];
    xu[3] = ibsulimits_.xu[3];

    z1 = xl[0] + x[0]*(xu[0] - xl[0]);
    z2 = xl[1] + x[1]*(xu[1] - xl[1]);
    z3 = xl[2] + x[2]*(xu[2] - xl[2]);
    z4 = xl[3] + x[3]*(xu[3] - xl[3]);

    c           = z1;
    x3          = z2;
    wy          = z3;
    x2          = z4;

    x1    = wy/x2/x3;
    aaa   = x1*x3+x2-c*(x1*x3-x2);
    s     = s0*wy;
    cti   = 1.0-(1.0-c)*2.0*x1*x3/aaa;
    sqs   = sqrt(s);
// anti-lepton:
    p[1][0] = sqs0*x1*x2*x3/aaa;
    p[1][1] = 0.0;
    p[1][2] = sqs/2.0*sqrt(1.0-cti*cti);
    p[1][3] = sqs0*x1*x2*x3/aaa*c;
// lepton:
    p[0][0] = sqs0/2.0*(x1*x3+x2)-sqs0*x1*x2*x3/aaa;
    p[0][1] = 0.0;
    p[0][2] = -sqs/2.0*sqrt(1.0-cti*cti);
    p[0][3] = sqrt(abs(p[0][0]*p[0][0] - p[1][2]*p[1][2]));
// photon:
    p[2][1] = 0.0;
    p[2][2] = 0.0;
    p[2][3] = 0.0;
    p[2][0] = 0.0;
}
//--------------------------------------------------------------------------------------------------
int main(void) {
    int wgt;
    long   loop;
    double MCresult,MCerror,MCwt;
    double branch[8];
    TPSEMAR *PseRan = new TPSEMAR();   // Create random number generator
#if IEXPLSOBR == 1
        TFDISTR *Density1 =        new TFDISTR(1);   // Create integrand function
        TFOAM   *FoamSobr = new TFOAM("FoamSobr");   // Create Simulator
//--------------------------------------------------------------------------------------------------
        FoamSobr->SetnDim(           nDimSobr);
        FoamSobr->SetkDim(           kDimSobr);
        FoamSobr->SetnCells(       nCellsSobr);
        FoamSobr->SetnSampl(       nSamplSobr);
        FoamSobr->SetnBin(           nBinSobr);
        FoamSobr->SetEvPerBin(   EvPerBinSobr);
        FoamSobr->SetOptRej(           OptRej);
        FoamSobr->SetOptDrive(       OptDrive);
        FoamSobr->SetOptPeek(         OptPeek);
        FoamSobr->SetOptEdge(         OptEdge);
        FoamSobr->SetOptOrd(           OptOrd);
        FoamSobr->SetOptMCell(       OptMCell);
        FoamSobr->SetMaxWtRej(   MaxWtRejSobr);
        FoamSobr->SetChat(               Chat);
        FoamSobr->Initialize(PseRan, Density1);
//--------------------------------------------------------------------------------------------------
        ifstream fin0 ("branch.dat", ios::in);
        fin0 >> branch[0];
	ofstream branch_sobr("branch_sobr.dat");
	branch_sobr << setprecision(10) << branch[0];
        TFile RootSobrFile("FoamSobr.root","RECREATE","Sobr");
        FoamSobr->Write("FoamSobr");    // Writing Foam on the disk
	remove("branch.dat");
#endif
#if IEXPLVIRT == 1
        TFDISTR *Density2 =        new TFDISTR(2);   // Create integrand function
        TFOAM   *FoamVirt = new TFOAM("FoamVirt");   // Create Simulator
//--------------------------------------------------------------------------------------------------
        FoamVirt->SetnDim(           nDimVirt);
        FoamVirt->SetkDim(           kDimVirt);
        FoamVirt->SetnCells(       nCellsVirt);
        FoamVirt->SetnSampl(       nSamplVirt);
        FoamVirt->SetnBin(           nBinVirt);
        FoamVirt->SetEvPerBin(   EvPerBinVirt);
        FoamVirt->SetOptRej(           OptRej);
        FoamVirt->SetOptDrive(       OptDrive);
        FoamVirt->SetOptPeek(         OptPeek);
        FoamVirt->SetOptEdge(         OptEdge);
        FoamVirt->SetOptOrd(           OptOrd);
        FoamVirt->SetOptMCell(       OptMCell);
        FoamVirt->SetMaxWtRej(   MaxWtRejVirt);
        FoamVirt->SetChat(               Chat);
        FoamVirt->Initialize(PseRan, Density2);
//--------------------------------------------------------------------------------------------------
        ifstream fin1 ("branch.dat", ios::in); 
        fin1 >> branch[1];
	ofstream branch_virt("branch_virt.dat");
	branch_virt << setprecision(10) << branch[1];
        TFile RootVirtFile("FoamVirt.root","RECREATE","Virt");
        FoamVirt->Write("FoamVirt");    // Writing Foam on the disk
	remove("branch.dat");
#endif
#if IEXPLBRDQ == 1
        TFDISTR *Density3        = new TFDISTR(3);   // Create integrand function
        TFOAM   *FoamBrdq = new TFOAM("FoamBrdq");   // Create Simulator
//--------------------------------------------------------------------------------------------------
        FoamBrdq->SetnDim(           nDimBrdq);
        FoamBrdq->SetkDim(           kDimBrdq);
        FoamBrdq->SetnCells(       nCellsBrdq);
        FoamBrdq->SetnSampl(       nSamplBrdq);
        FoamBrdq->SetnBin(           nBinBrdq);
        FoamBrdq->SetEvPerBin(   EvPerBinBrdq);
        FoamBrdq->SetOptRej(           OptRej);
        FoamBrdq->SetOptDrive(       OptDrive);
        FoamBrdq->SetOptPeek(         OptPeek);
        FoamBrdq->SetOptEdge(         OptEdge);
        FoamBrdq->SetOptOrd(           OptOrd);
        FoamBrdq->SetOptMCell(       OptMCell);
        FoamBrdq->SetMaxWtRej(   MaxWtRejBrdq);
        FoamBrdq->SetChat(               Chat);
        FoamBrdq->Initialize(PseRan, Density3);
//--------------------------------------------------------------------------------------------------
        ifstream fin2 ("branch.dat", ios::in); 
        fin2 >> branch[2];
	ofstream branch_brdq("branch_brdq.dat");
	branch_brdq << setprecision(10) << branch[2];
        TFile RootBrdqFile("FoamBrdq.root","RECREATE","Brdq");
        FoamBrdq->Write("FoamBrdq");    // Writing Foam on the disk
	remove("branch.dat");
#endif
#if IEXPLHARD == 1
        TFDISTR *Density4        = new TFDISTR(4);   // Create integrand function
        TFOAM   *FoamHard = new TFOAM("FoamHard");   // Create Simulator
//--------------------------------------------------------------------------------------------------
        FoamHard->SetnDim(           nDimHard);
        FoamHard->SetkDim(           kDimHard);
        FoamHard->SetnCells(       nCellsHard);
        FoamHard->SetnSampl(       nSamplHard);
        FoamHard->SetnBin(           nBinHard);
        FoamHard->SetEvPerBin(   EvPerBinHard);
        FoamHard->SetOptRej(           OptRej);
        FoamHard->SetOptDrive(       OptDrive);
        FoamHard->SetOptPeek(         OptPeek);
        FoamHard->SetOptEdge(         OptEdge);
        FoamHard->SetOptOrd(           OptOrd);
        FoamHard->SetOptMCell(       OptMCell);
        FoamHard->SetMaxWtRej(   MaxWtRejHard);
        FoamHard->SetChat(               Chat);
        FoamHard->Initialize(PseRan, Density4);
//--------------------------------------------------------------------------------------------------
        ifstream fin3 ("branch.dat", ios::in); 
        fin3 >> branch[3];
	ofstream branch_hard("branch_hard.dat");
	branch_hard << setprecision(10) << branch[3];
        TFile RootHardFile("FoamHard.root","RECREATE","Hard");
        FoamHard->Write("FoamHard");    // Writing Foam on the disk
	remove("branch.dat");
#endif
#if INVBREM == 1
#if IEXPLIBBR == 1
        TFDISTR *Density5 =        new TFDISTR(5);   // Create integrand function
        TFOAM   *FoamIbbr = new TFOAM("FoamIbbr");   // Create Simulator
//--------------------------------------------------------------------------------------------------
        FoamIbbr->SetnDim(           nDimIbbr);
        FoamIbbr->SetkDim(           kDimIbbr);
        FoamIbbr->SetnCells(       nCellsIbbr);
        FoamIbbr->SetnSampl(       nSamplIbbr);
        FoamIbbr->SetnBin(           nBinIbbr);
        FoamIbbr->SetEvPerBin(   EvPerBinIbbr);
        FoamIbbr->SetOptRej(           OptRej);
        FoamIbbr->SetOptDrive(       OptDrive);
        FoamIbbr->SetOptPeek(         OptPeek);
        FoamIbbr->SetOptEdge(         OptEdge);
        FoamIbbr->SetOptOrd(           OptOrd);
        FoamIbbr->SetOptMCell(       OptMCell);
        FoamIbbr->SetMaxWtRej(   MaxWtRejIbbr);
        FoamIbbr->SetChat(               Chat);
        FoamIbbr->Initialize(PseRan, Density5);
//--------------------------------------------------------------------------------------------------
        ifstream fin4 ("branch.dat", ios::in);
        fin4 >> branch[4];
	ofstream branch_ibbr("branch_ibbr.dat");
	branch_ibbr << setprecision(10) << branch[4];
        TFile RootIbbrFile("FoamIbbr.root","RECREATE","Ibbr");
        FoamIbbr->Write("FoamIbbr");    // Writing Foam on the disk
	remove("branch.dat");
#endif
#if IEXPLIBS1 == 1
        TFDISTR *Density6 =        new TFDISTR(6);   // Create integrand function
        TFOAM   *FoamIbs1 = new TFOAM("FoamIbs1");   // Create Simulator
//--------------------------------------------------------------------------------------------------
        FoamIbs1->SetnDim(           nDimIbs1);
        FoamIbs1->SetkDim(           kDimIbs1);
        FoamIbs1->SetnCells(       nCellsIbs1);
        FoamIbs1->SetnSampl(       nSamplIbs1);
        FoamIbs1->SetnBin(           nBinIbs1);
        FoamIbs1->SetEvPerBin(   EvPerBinIbs1);
        FoamIbs1->SetOptRej(           OptRej);
        FoamIbs1->SetOptDrive(       OptDrive);
        FoamIbs1->SetOptPeek(         OptPeek);
        FoamIbs1->SetOptEdge(         OptEdge);
        FoamIbs1->SetOptOrd(           OptOrd);
        FoamIbs1->SetOptMCell(       OptMCell);
        FoamIbs1->SetMaxWtRej(   MaxWtRejIbs1);
        FoamIbs1->SetChat(               Chat);
        FoamIbs1->Initialize(PseRan, Density6);
//--------------------------------------------------------------------------------------------------
        ifstream fin5 ("branch.dat", ios::in);
        fin5 >> branch[5];
	ofstream branch_ibs1("branch_ibs1.dat");
	branch_ibs1 << setprecision(10) << branch[5];
        TFile RootIbs1File("FoamIbs1.root","RECREATE","Ibs1");
        FoamIbs1->Write("FoamIbs1");    // Writing Foam on the disk
	remove("branch.dat");
#endif
#if IEXPLIBS2 == 1
        TFDISTR *Density7 =        new TFDISTR(7);   // Create integrand function
        TFOAM   *FoamIbs2 = new TFOAM("FoamIbs2");   // Create Simulator
//--------------------------------------------------------------------------------------------------
        FoamIbs2->SetnDim(           nDimIbs2);
        FoamIbs2->SetkDim(           kDimIbs2);
        FoamIbs2->SetnCells(       nCellsIbs2);
        FoamIbs2->SetnSampl(       nSamplIbs2);
        FoamIbs2->SetnBin(           nBinIbs2);
        FoamIbs2->SetEvPerBin(   EvPerBinIbs2);
        FoamIbs2->SetOptRej(           OptRej);
        FoamIbs2->SetOptDrive(       OptDrive);
        FoamIbs2->SetOptPeek(         OptPeek);
        FoamIbs2->SetOptEdge(         OptEdge);
        FoamIbs2->SetOptOrd(           OptOrd);
        FoamIbs2->SetOptMCell(       OptMCell);
        FoamIbs2->SetMaxWtRej(   MaxWtRejIbs2);
        FoamIbs2->SetChat(               Chat);
        FoamIbs2->Initialize(PseRan, Density7);
//--------------------------------------------------------------------------------------------------
        ifstream fin6 ("branch.dat", ios::in);
        fin6 >> branch[6];
	ofstream branch_ibs2("branch_ibs2.dat");
	branch_ibs2 << setprecision(10) << branch[6];
        TFile RootIbs2File("FoamIbs2.root","RECREATE","Ibs2");
        FoamIbs2->Write("FoamIbs2");    // Writing Foam on the disk
	remove("branch.dat");
#endif
#endif
#if IEXPLBORN == 1
        TFDISTR *Density8 =        new TFDISTR(8);   // Create integrand function
        TFOAM   *FoamBorn = new TFOAM("FoamBorn");   // Create Simulator
//--------------------------------------------------------------------------------------------------
        FoamBorn->SetnDim(           nDimBorn);
        FoamBorn->SetkDim(           kDimBorn);
        FoamBorn->SetnCells(       nCellsBorn);
        FoamBorn->SetnSampl(       nSamplBorn);
        FoamBorn->SetnBin(           nBinBorn);
        FoamBorn->SetEvPerBin(   EvPerBinBorn);
        FoamBorn->SetOptRej(           OptRej);
        FoamBorn->SetOptDrive(       OptDrive);
        FoamBorn->SetOptPeek(         OptPeek);
        FoamBorn->SetOptEdge(         OptEdge);
        FoamBorn->SetOptOrd(           OptOrd);
        FoamBorn->SetOptMCell(       OptMCell);
        FoamBorn->SetMaxWtRej(   MaxWtRejBorn);
        FoamBorn->SetChat(               Chat);
        FoamBorn->Initialize(PseRan, Density8);
//--------------------------------------------------------------------------------------------------
        ifstream fin7 ("branch.dat", ios::in);
        fin7 >> branch[7];
	ofstream branch_born("branch_born.dat");
	branch_born << setprecision(10) << branch[7];
        TFile RootBornFile("FoamBorn.root","RECREATE","Born");
        FoamBorn->Write("FoamBorn");    // Writing Foam on the disk
	remove("branch.dat");
#endif
        cout << "====== Initialization done, entering MC loop" << endl;
#if IEXPLSOBR == 0
        TFile fileSobr("FoamSobr.root"); // connect disk file
//--------------------------------------------------------------------------------------------------
        TFOAM *FoamSobr = (TFOAM*)fileSobr.Get("FoamSobr"); // find object
        FoamSobr->SetMaxWtRej(       MaxWtRejSobr);
        FoamSobr->LinkCells(); // restore pointers of the binary tree of cells
        FoamSobr->GetTotDim();
#endif
#if IEXPLVIRT == 0
        TFile fileVirt("FoamVirt.root"); // connect disk file
//--------------------------------------------------------------------------------------------------
        TFOAM *FoamVirt = (TFOAM*)fileVirt.Get("FoamVirt"); // find object
        FoamVirt->SetMaxWtRej(       MaxWtRejVirt);
        FoamVirt->LinkCells(); // restore pointers of the binary tree of cells
        FoamVirt->GetTotDim();
#endif
#if IEXPLBRDQ == 0
        TFile fileBrdq("FoamBrdq.root"); // connect disk file
//--------------------------------------------------------------------------------------------------
        TFOAM *FoamBrdq = (TFOAM*)fileBrdq.Get("FoamBrdq"); // find object
        FoamBrdq->SetMaxWtRej(       MaxWtRejBrdq);
        FoamBrdq->LinkCells(); // restore pointers of the binary tree of cells
        FoamBrdq->GetTotDim();
#endif
#if IEXPLHARD == 0
        TFile fileHard("FoamHard.root"); // connect disk file
//--------------------------------------------------------------------------------------------------
        TFOAM *FoamHard = (TFOAM*)fileHard.Get("FoamHard"); // find object
        FoamHard->SetMaxWtRej(       MaxWtRejHard);
        FoamHard->LinkCells(); // restore pointers of the binary tree of cells
        FoamHard->GetTotDim();
#endif
#if INVBREM == 1
#if IEXPLIBBR == 0
        TFile fileIbbr("FoamIbbr.root"); // connect disk file
//--------------------------------------------------------------------------------------------------
        TFOAM *FoamIbbr = (TFOAM*)fileIbbr.Get("FoamIbbr"); // find object
        FoamIbbr->SetMaxWtRej(       MaxWtRejIbbr);
        FoamIbbr->LinkCells(); // restore pointers of the binary tree of cells
        FoamIbbr->GetTotDim();
#endif
#if IEXPLIBS1 == 0
        TFile fileIbs1("FoamIbs1.root"); // connect disk file
//--------------------------------------------------------------------------------------------------
        TFOAM *FoamIbs1 = (TFOAM*)fileIbs1.Get("FoamIbs1"); // find object
        FoamIbs1->SetMaxWtRej(       MaxWtRejIbs1);
        FoamIbs1->LinkCells(); // restore pointers of the binary tree of cells
        FoamIbs1->GetTotDim();
#endif
#if IEXPLIBS2 == 0
        TFile fileIbs2("FoamIbs2.root"); // connect disk file
//--------------------------------------------------------------------------------------------------
        TFOAM *FoamIbs2 = (TFOAM*)fileIbs2.Get("FoamIbs2"); // find object
        FoamIbs2->SetMaxWtRej(       MaxWtRejIbs2);
        FoamIbs2->LinkCells(); // restore pointers of the binary tree of cells
        FoamIbs2->GetTotDim();
#endif
#endif
#if IEXPLBORN == 0
        TFile fileBorn("FoamBorn.root"); // connect disk file
//--------------------------------------------------------------------------------------------------
        TFOAM *FoamBorn = (TFOAM*)fileBorn.Get("FoamBorn"); // find object
        FoamBorn->SetMaxWtRej(       MaxWtRejBorn);
        FoamBorn->LinkCells(); // restore pointers of the binary tree of cells
        FoamBorn->GetTotDim();
#endif
//--------------------------------------------------------------------------------------------------
    ifstream finsobr ("branch_sobr.dat", ios::in); 
    finsobr >> branch[0];
    ifstream finvirt ("branch_virt.dat", ios::in); 
    finvirt >> branch[1];
    ifstream finbrdq ("branch_brdq.dat", ios::in); 
    finbrdq >> branch[2];
    ifstream finhard ("branch_hard.dat", ios::in); 
    finhard >> branch[3];
    ifstream finibbr ("branch_ibbr.dat", ios::in); 
    finibbr >> branch[4];
    ifstream finibs1 ("branch_ibs1.dat", ios::in); 
    finibs1 >> branch[5];
    ifstream finibs2 ("branch_ibs2.dat", ios::in); 
    finibs2 >> branch[6];
    ifstream finborn ("branch_born.dat", ios::in); 
    finborn >> branch[7];
#if INVBREM == 0
        branch[4] = 0.0;
        branch[5] = 0.0;
        branch[6] = 0.0;
#endif
//--------------------------------------------------------------------------------------------------
    cout << "-------------------" << endl;
    cout << "Branching weights:" << endl;
    cout << "Sobr: " << branch[0] << endl;
    cout << "Virt: " << branch[1] << endl;
    cout << "Brdq: " << branch[2] << endl;
    cout << "Hard: " << branch[3] << endl;
    cout << "Ibbr: " << branch[4] << endl;
    cout << "Ibs1: " << branch[5] << endl;
    cout << "Ibs2: " << branch[6] << endl;
    cout << "Born: " << branch[7] << endl;
    cout << "-------------------" << endl;
//--------------------------------------------------------------------------------------------------
    double *MCvectSobr = new double[TotDimSobr];   // vector generated in the MC run
    double *MCvectVirt = new double[TotDimVirt];   // vector generated in the MC run
    double *MCvectBrdq = new double[TotDimBrdq];   // vector generated in the MC run
    double *MCvectHard = new double[TotDimHard];   // vector generated in the MC run
    double *MCvectIbbr = new double[TotDimIbbr];   // vector generated in the MC run
    double *MCvectIbs1 = new double[TotDimIbs1];   // vector generated in the MC run
    double *MCvectIbs2 = new double[TotDimIbs2];   // vector generated in the MC run
    double *MCvectBorn = new double[TotDimBorn];   // vector generated in the MC run
//--------------------------------------------------------------------------------------------------
    SigmaSobr = branch[0];
    SigmaVirt = branch[1];
    SigmaBrdq = branch[2];
    SigmaHard = branch[3];
    SigmaIbbr = branch[4];
    SigmaIbs1 = branch[5];
    SigmaIbs2 = branch[6];
    SigmaBorn = branch[7];
    double rBr;
    double SigmaTot  = SigmaSobr+SigmaVirt+SigmaBrdq+SigmaHard+SigmaIbbr+SigmaIbs1+SigmaIbs2;
    double BrSobr    = SigmaSobr/SigmaTot;
    double BrVirt    = (SigmaSobr+SigmaVirt)/SigmaTot;
    double BrBrdq    = (SigmaSobr+SigmaVirt+SigmaBrdq)/SigmaTot;
    double BrHard    = (SigmaSobr+SigmaVirt+SigmaBrdq+SigmaHard)/SigmaTot;
    double BrIbbr    = (SigmaSobr+SigmaVirt+SigmaBrdq+SigmaHard+SigmaIbbr)/SigmaTot;
    double BrIbs1    = (SigmaSobr+SigmaVirt+SigmaBrdq+SigmaHard+SigmaIbbr+SigmaIbs1)/SigmaTot;
//--------------------------------------------------------------------------------------------------
    double pSobr[3][4];
    double pVirt[3][4];
    double pBrdq[3][4];
    double pHard[3][4];
    double pIbbr[3][4];
    double pIbs1[3][4];
    double pIbs2[3][4];
    double pBorn[3][4];
//--------------------------------------------------------------------------------------------------
    int iSobrPWt1  = 0;
    int iSobrPWtN1 = 0;
    int iVirtPWt1  = 0;
    int iVirtPWtN1 = 0;
    int iBrdqPWt1  = 0;
    int iBrdqPWtN1 = 0;
    int iHardPWt1  = 0;
    int iHardPWtN1 = 0;
    int iIbbrPWt1  = 0;
    int iIbbrPWtN1 = 0;
    int iIbs1PWt1  = 0;
    int iIbs1PWtN1 = 0;
    int iIbs2PWt1  = 0;
    int iIbs2PWtN1 = 0;
    int iBornPWt1  = 0;
    int iBornPWtN1 = 0;
//--------------------------------------------------------------------------------------------------
    int iSobrNWt1  = 0;
    int iSobrNWtN1 = 0;
    int iVirtNWt1  = 0;
    int iVirtNWtN1 = 0;
    int iBrdqNWt1  = 0;
    int iBrdqNWtN1 = 0;
    int iHardNWt1  = 0;
    int iHardNWtN1 = 0;
    int iIbbrNWt1  = 0;
    int iIbbrNWtN1 = 0;
    int iIbs1NWt1  = 0;
    int iIbs1NWtN1 = 0;
    int iIbs2NWt1  = 0;
    int iIbs2NWtN1 = 0;
    int iBornNWt1  = 0;
    int iBornNWtN1 = 0;
//--------------------------------------------------------------------------------------------------
    int iNWt1  = 0;
    int iNWtN1 = 0;
    int iPWt1  = 0;
    int iPWtN1 = 0;
    ofstream output_tot;
    ofstream output_brn;
    output_tot.open("../nc_dy_foam_evaluation/events_tot.dat");
    output_brn.open("../nc_dy_foam_evaluation/events_brn.dat");
//--------------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------------
    for (loop = 0; loop < NevTot; loop++) {
        rBr = (double) random()/RAND_MAX;
//--------------------------------------------------------------------------------------------------
    	if (rBr <= BrSobr) {
            FoamSobr->MakeEvent();   // generate Soft + Born MC event
            FoamSobr->GetMCvect(MCvectSobr);
            MCwt = FoamSobr->GetMCwt();
	    get_vectors_soft(MCvectSobr, pSobr);
            if (MCwt == 1.0) {
		wgt = weight_.wgt;
                if (wgt > 0.0) iSobrPWt1++;
                if (wgt < 0.0) iSobrNWt1++;
		output_tot << SW3 << wgt << "  ";
	    	for (int i = 0; i < 3; i++) {
		    for (int k = 0; k < 4; k++) {
		    	output_tot << SP12 << pSobr[i][k] << " ";
		    }
            	}
		output_tot << endl;
            }
            if (MCwt > 1.0) {
		wgt = weight_.wgt;
                if (wgt > 0.0) iSobrPWtN1++;
                if (wgt < 0.0) { iSobrNWtN1++; MCwt = -MCwt; }
            	cout << "Sobr " << "MCwt   = " << SP12 << MCwt << ",  ";
            	cout << "MCvect = ";
	    	for (int i = 0; i < 3; i++) {
		    for (int k = 0; k < 4; k++) {
		    	cout << SP12 << pSobr[i][k] << " ";
		    }
            	}
		cout << endl;
            }
        }
//--------------------------------------------------------------------------------------------------
    	else if (rBr <= BrVirt) {
            FoamVirt->MakeEvent();   // generate Virt MC event
            FoamVirt->GetMCvect(MCvectVirt);
            MCwt = FoamVirt->GetMCwt();
	    get_vectors_soft(MCvectVirt, pVirt);
            if (MCwt == 1.0) {
		wgt = weight_.wgt;
                if (wgt > 0.0) iVirtPWt1++;
                if (wgt < 0.0) iVirtNWt1++;
		output_tot << SW3 << wgt << "  ";
	    	for (int i = 0; i < 3; i++) {
		    for (int k = 0; k < 4; k++) {
		    	output_tot << SP12 << pVirt[i][k] << " ";
		    }
            	}
		output_tot << endl;
            }
            if (MCwt > 1.0) {
		wgt = weight_.wgt;
                if (wgt > 0.0) iVirtPWtN1++;
                if (wgt < 0.0) { iVirtNWtN1++; MCwt = -MCwt; }
            	cout << "Virt " << "MCwt   = " << SP12 << MCwt << ",  ";
            	cout << "MCvect = ";
	    	for (int i = 0; i < 3; i++) {
		    for (int k = 0; k < 4; k++) {
		    	cout << SP12 << pVirt[i][k] << " ";
		    }
            	}
		cout << endl;
            }
    	}
//--------------------------------------------------------------------------------------------------
    	else if (rBr <= BrBrdq) {
            FoamBrdq->MakeEvent();   // generate Brdq MC event
            FoamBrdq->GetMCvect(MCvectBrdq);
            MCwt = FoamBrdq->GetMCwt();
	    get_vectors_soft(MCvectBrdq, pBrdq);
            if (MCwt == 1.0) {
		wgt = weight_.wgt;
                if (wgt > 0.0) iBrdqPWt1++;
                if (wgt < 0.0) iBrdqNWt1++;
		output_tot << SW3 << wgt << "  ";
	    	for (int i = 0; i < 3; i++) {
		    for (int k = 0; k < 4; k++) {
		    	output_tot << SP12 << pBrdq[i][k] << " ";
		    }
            	}
		output_tot << endl;
            }
            if (MCwt > 1.0) {
		wgt = weight_.wgt;
                if (wgt > 0.0) iBrdqPWtN1++;
                if (wgt < 0.0) { iBrdqNWtN1++; MCwt = -MCwt; }
            	cout << "Brdq " << "MCwt   = " << SP12 << MCwt << ",  ";
            	cout << "MCvect = ";
	    	for (int i = 0; i < 3; i++) {
		    for (int k = 0; k < 4; k++) {
		    	cout << SP12 << pBrdq[i][k] << " ";
		    }
            	}
		cout << endl;
            }
    	}
//--------------------------------------------------------------------------------------------------
    	else if (rBr <= BrHard) {
            FoamHard->MakeEvent();   // generate Hard MC event
            FoamHard->GetMCvect(MCvectHard);
            MCwt = FoamHard->GetMCwt();
	    get_vectors_hard(MCvectHard, pHard);
            if (MCwt == 1.0) {
		wgt = weight_.wgt;
                if (wgt > 0.0) iHardPWt1++;
                if (wgt < 0.0) iHardNWt1++;
		output_tot << SW3 << wgt << "  ";
	    	for (int i = 0; i < 3; i++) {
		    for (int k = 0; k < 4; k++) {
		    	output_tot << SP12 << pHard[i][k] << " ";
		    }
            	}
		output_tot << endl;
            }
            if (MCwt > 1.0) {
		wgt = weight_.wgt;
                if (wgt > 0.0) iHardPWtN1++;
                if (wgt < 0.0) { iHardNWtN1++; MCwt = -MCwt; }
            	cout << "Hard " << "MCwt   = " << SP12 << MCwt << ",  ";
            	cout << "MCvect = ";
	    	for (int i = 0; i < 3; i++) {
	            for (int k = 0; k < 4; k++) {
		    	cout << SP12 << pHard[i][k] << " ";
		    }
            	}
		cout << endl;
            }
    	}
//--------------------------------------------------------------------------------------------------
#if INVBREM == 1
    	else if (rBr <= BrIbbr) {
            FoamIbbr->MakeEvent();   // generate Ibbr MC event
            FoamIbbr->GetMCvect(MCvectIbbr);
            MCwt = FoamIbbr->GetMCwt();
	    get_vectors_ibbr(MCvectIbbr, pIbbr);
            if (MCwt == 1.0) {
		wgt = weight_.wgt;
                if (wgt > 0.0) iIbbrPWt1++;
                if (wgt < 0.0) iIbbrNWt1++;
		output_tot << SW3 << wgt << "  ";
	    	for (int i = 0; i < 3; i++) {
		    for (int k = 0; k < 4; k++) {
		    	output_tot << SP12 << pIbbr[i][k] << " ";
		    }
            	}
		output_tot << endl;
            }
            if (MCwt > 1.0) {
		wgt = weight_.wgt;
                if (wgt > 0.0) iIbbrPWtN1++;
                if (wgt < 0.0) { iIbbrNWtN1++; MCwt = -MCwt; }
            	cout << "Ibbr " << "MCwt   = " << SP12 << MCwt << ",  ";
            	cout << "MCvect = ";
	    	for (int i = 0; i < 3; i++) {
	            for (int k = 0; k < 4; k++) {
		    	cout << SP12 << pIbbr[i][k] << " ";
		    }
            	}
		cout << endl;
            }
    	}
//--------------------------------------------------------------------------------------------------
    	else if (rBr <= BrIbs1) {
            FoamIbs1->MakeEvent();   // generate Ibs1 MC event
            FoamIbs1->GetMCvect(MCvectIbs1);
            MCwt = FoamIbs1->GetMCwt();
	    get_vectors_ibsu(MCvectIbs1, pIbs1);
            if (MCwt == 1.0) {
		wgt = weight_.wgt;
                if (wgt > 0.0) iIbs1PWt1++;
                if (wgt < 0.0) iIbs1NWt1++;
		output_tot << SW3 << wgt << "  ";
	    	for (int i = 0; i < 3; i++) {
		    for (int k = 0; k < 4; k++) {
		    	output_tot << SP12 << pIbs1[i][k] << " ";
		    }
            	}
		output_tot << endl;
            }
            if (MCwt > 1.0) {
		wgt = weight_.wgt;
                if (wgt > 0.0) iIbs1PWtN1++;
                if (wgt < 0.0) { iIbs1NWtN1++; MCwt = -MCwt; }
            	cout << "Ibs1 " << "MCwt   = " << SP12 << MCwt << ",  ";
            	cout << "MCvect = ";
	    	for (int i = 0; i < 3; i++) {
	            for (int k = 0; k < 4; k++) {
		    	cout << SP12 << pIbs1[i][k] << " ";
		    }
            	}
		cout << endl;
            }
    	}
//--------------------------------------------------------------------------------------------------
    	else {
            FoamIbs2->MakeEvent();   // generate Ibs2 MC event
            FoamIbs2->GetMCvect(MCvectIbs2);
            MCwt = FoamIbs2->GetMCwt();
	    get_vectors_ibsu(MCvectIbs2, pIbs2);
            if (MCwt == 1.0) {
		wgt = weight_.wgt;
                if (wgt > 0.0) iIbs2PWt1++;
                if (wgt < 0.0) iIbs2NWt1++;
		output_tot << SW3 << wgt << "  ";
	    	for (int i = 0; i < 3; i++) {
		    for (int k = 0; k < 4; k++) {
		    	output_tot << SP12 << pIbs2[i][k] << " ";
		    }
            	}
		output_tot << endl;
            }
            if (MCwt > 1.0) {
		wgt = weight_.wgt;
                if (wgt > 0.0) iIbs2PWtN1++;
                if (wgt < 0.0) { iIbs2NWtN1++; MCwt = -MCwt; }
            	cout << "Ibs2 " << "MCwt   = " << SP12 << MCwt << ",  ";
            	cout << "MCvect = ";
	    	for (int i = 0; i < 3; i++) {
	            for (int k = 0; k < 4; k++) {
		    	cout << SP12 << pIbs2[i][k] << " ";
		    }
            	}
		cout << endl;
            }
    	}
#endif
//--------------------------------------------------------------------------------------------------
    }
//--------------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------------
    for (loop = 0; loop < NevBrn; loop++) {
//--------------------------------------------------------------------------------------------------
        FoamBorn->MakeEvent();   // generate Born MC event
        FoamBorn->GetMCvect(MCvectBorn);
        MCwt = FoamBorn->GetMCwt();
	get_vectors_soft(MCvectBorn, pBorn);
        if (MCwt == 1.0) {
	    wgt = weight_.wgt;
            if (wgt > 0.0) iBornPWt1++;
            if (wgt < 0.0) iBornNWt1++;
	    output_brn << SW3 << wgt << "  ";
	    for (int i = 0; i < 3; i++) {
		for (int k = 0; k < 4; k++) {
		    output_brn << SP12 << pBorn[i][k] << " ";
		}
            }
	    output_brn << endl;
        }
        if (MCwt > 1.0) {
	    wgt = weight_.wgt;
            if (wgt > 0.0) iBornPWtN1++;
            if (wgt < 0.0) { iBornNWtN1++; MCwt = -MCwt; }
            cout << "Born " << "MCwt   = " << SP12 << MCwt << ",  ";
            cout << "MCvect = ";
	    for (int i = 0; i < 3; i++) {
		for (int k = 0; k < 4; k++) {
		    cout << SP12 << pBorn[i][k] << " ";
		}
            }
	    cout << endl;
        }
    }
//--------------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------------
    output_tot.close();
    output_brn.close();
    double SobrF = 0.0;
    double VirtF = 0.0;
    double BrdqF = 0.0;
    double HardF = 0.0;
    double IbbrF = 0.0;
    double Ibs1F = 0.0;
    double Ibs2F = 0.0;
    double BornF = 0.0;
    iPWt1  = iSobrPWt1+iVirtPWt1+iBrdqPWt1+iHardPWt1+iIbbrPWt1+iIbs1PWt1+iIbs2PWt1;
    iNWt1  = iSobrNWt1+iVirtNWt1+iBrdqNWt1+iHardNWt1+iIbbrNWt1+iIbs1NWt1+iIbs2NWt1;
    iPWtN1 = iSobrPWtN1+iVirtPWtN1+iBrdqPWtN1+iHardPWtN1+iIbbrPWtN1+iIbs1PWtN1+iIbs2PWtN1;
    iNWtN1 = iSobrNWtN1+iVirtNWtN1+iBrdqNWtN1+iHardNWtN1+iIbbrNWtN1+iIbs1NWtN1+iIbs2NWtN1;
    if ((iSobrPWt1+iSobrNWt1) > 0) SobrF = (double) (iSobrPWt1-iSobrNWt1)/(iSobrPWt1+iSobrNWt1);
    if ((iVirtPWt1+iVirtNWt1) > 0) VirtF = (double) (iVirtPWt1-iVirtNWt1)/(iVirtPWt1+iVirtNWt1);
    if ((iBrdqPWt1+iBrdqNWt1) > 0) BrdqF = (double) (iBrdqPWt1-iBrdqNWt1)/(iBrdqPWt1+iBrdqNWt1);
    if ((iHardPWt1+iHardNWt1) > 0) HardF = (double) (iHardPWt1-iHardNWt1)/(iHardPWt1+iHardNWt1);
    if ((iIbbrPWt1+iIbbrNWt1) > 0) IbbrF = (double) (iIbbrPWt1-iIbbrNWt1)/(iIbbrPWt1+iIbbrNWt1);
    if ((iIbs1PWt1+iIbs1NWt1) > 0) Ibs1F = (double) (iIbs1PWt1-iIbs1NWt1)/(iIbs1PWt1+iIbs1NWt1);
    if ((iIbs2PWt1+iIbs2NWt1) > 0) Ibs2F = (double) (iIbs2PWt1-iIbs2NWt1)/(iIbs2PWt1+iIbs2NWt1);
    if ((iBornPWt1+iBornNWt1) > 0) BornF = (double) (iBornPWt1-iBornNWt1)/(iBornPWt1+iBornNWt1);
//--------------------------------------------------------------------------------------------------
    cout << "====== Events generated, entering Finalize" << endl;
//--------------------------------------------------------------------------------------------------
    FoamSobr->GetIntegMC(MCresult, MCerror);  // get MC integral
    SigmaSobr = SobrF*MCresult;
    double ErrSobr = MCerror;
    cout << "================================================================" << endl;
    cout << "        MCresult (Sobr) = " << SigmaSobr << " +- " << ErrSobr << " RelErr = " << ErrSobr/SigmaSobr << endl;
    delete [] MCvectSobr;
//--------------------------------------------------------------------------------------------------
    FoamVirt->GetIntegMC(MCresult, MCerror);  // get MC integral
    SigmaVirt = VirtF*MCresult;
    double ErrVirt = MCerror;
    cout << "================================================================" << endl;
    cout << "        MCresult (Virt) = " << SigmaVirt << " +- " << ErrVirt << " RelErr = " << ErrVirt/SigmaVirt << endl;
    delete [] MCvectVirt;
//--------------------------------------------------------------------------------------------------
    FoamBrdq->GetIntegMC(MCresult, MCerror);  // get MC integral
    SigmaBrdq = BrdqF*MCresult;
    double ErrBrdq = MCerror;
    cout << "================================================================" << endl;
    cout << "        MCresult (Brdq) = " << SigmaBrdq << " +- " << ErrBrdq << " RelErr = " << ErrBrdq/SigmaBrdq << endl;
    delete [] MCvectBrdq;
//--------------------------------------------------------------------------------------------------
    FoamHard->GetIntegMC(MCresult, MCerror);  // get MC integral
    SigmaHard = HardF*MCresult;
    double ErrHard = MCerror;
    cout << "================================================================" << endl;
    cout << "        MCresult (Hard) = " << SigmaHard << " +- " << ErrHard << " RelErr = " << ErrHard/SigmaHard << endl;
    delete [] MCvectHard;
//--------------------------------------------------------------------------------------------------
#if INVBREM == 1
    FoamIbbr->GetIntegMC(MCresult, MCerror);  // get MC integral
    SigmaIbbr = IbbrF*MCresult;
    double ErrIbbr = MCerror;
    cout << "================================================================" << endl;
    cout << "        MCresult (Ibbr) = " << SigmaIbbr << " +- " << ErrIbbr << " RelErr = " << ErrIbbr/SigmaIbbr << endl;
    delete [] MCvectIbbr;
//--------------------------------------------------------------------------------------------------
    FoamIbs1->GetIntegMC(MCresult, MCerror);  // get MC integral
    SigmaIbs1 = Ibs1F*MCresult;
    double ErrIbs1 = MCerror;
    cout << "================================================================" << endl;
    cout << "        MCresult (Ibs1) = " << SigmaIbs1 << " +- " << ErrIbs1 << " RelErr = " << ErrIbs1/SigmaIbs1 << endl;
    delete [] MCvectIbs1;
//--------------------------------------------------------------------------------------------------
    FoamIbs2->GetIntegMC(MCresult, MCerror);  // get MC integral
    SigmaIbs2 = Ibs2F*MCresult;
    double ErrIbs2 = MCerror;
    cout << "================================================================" << endl;
    cout << "        MCresult (Ibs2) = " << SigmaIbs2 << " +- " << ErrIbs2 << " RelErr = " << ErrIbs2/SigmaIbs2 << endl;
    delete [] MCvectIbs2;
#endif
//--------------------------------------------------------------------------------------------------
#if INVBREM == 0
    double ErrIbbr = 0.0;
    double ErrIbs1 = 0.0;
    double ErrIbs2 = 0.0;
#endif
    SigmaTot = SigmaSobr+SigmaVirt+SigmaBrdq+SigmaHard+SigmaIbbr+SigmaIbs1+SigmaIbs2;
    double ErrTot = sqrt(ErrSobr*ErrSobr
    +ErrVirt*ErrVirt
    +ErrBrdq*ErrBrdq
    +ErrHard*ErrHard
    +ErrIbbr*ErrIbbr
    +ErrIbs1*ErrIbs1
    +ErrIbs2*ErrIbs2);
    cout << "================================================================" << endl;
    cout << "        MCresult (Tot)  = " << SigmaTot << " +- " << ErrTot << " RelErr = " << ErrTot/SigmaTot << endl;
//--------------------------------------------------------------------------------------------------
    FoamBorn->GetIntegMC(MCresult, MCerror);  // get MC integral
    SigmaBorn = BornF*MCresult;
    double ErrBorn = MCerror;
    cout << "================================================================" << endl;
    cout << "        MCresult (Born) = " << SigmaBorn << " +- " << ErrBorn << " RelErr = " << ErrBorn/SigmaBorn << endl;
    cout << "================================================================" << endl;
    delete [] MCvectBorn;
    ofstream sigma_tot;
    ofstream sigma_brn;
    sigma_tot.open("../nc_dy_foam_evaluation/sigma_tot.dat");
    sigma_brn.open("../nc_dy_foam_evaluation/sigma_brn.dat");
    sigma_tot << setprecision(10) << SigmaTot  << endl;
    sigma_brn << setprecision(10) << SigmaBorn << endl;
    sigma_tot.close();
    sigma_brn.close();
//--------------------------------------------------------------------------------------------------
    cout << "Total number of Sobr events: " << iSobrPWt1 + iSobrPWtN1 << " pos. " << iSobrNWt1 + iSobrNWtN1 << " neg." << endl;
    cout << "Total number of Virt events: " << iVirtPWt1 + iVirtPWtN1 << " pos. " << iVirtNWt1 + iVirtNWtN1 << " neg." << endl;
    cout << "Total number of Brdq events: " << iBrdqPWt1 + iBrdqPWtN1 << " pos. " << iBrdqNWt1 + iBrdqNWtN1 << " neg." << endl;
    cout << "Total number of Hard events: " << iHardPWt1 + iHardPWtN1 << " pos. " << iHardNWt1 + iHardNWtN1 << " neg." << endl;
    cout << "Total number of Ibbr events: " << iIbbrPWt1 + iIbbrPWtN1 << " pos. " << iIbbrNWt1 + iIbbrNWtN1 << " neg." << endl;
    cout << "Total number of Ibs1 events: " << iIbs1PWt1 + iIbs1PWtN1 << " pos. " << iIbs1NWt1 + iIbs1NWtN1 << " neg." << endl;
    cout << "Total number of Ibs2 events: " << iIbs2PWt1 + iIbs2PWtN1 << " pos. " << iIbs2NWt1 + iIbs2NWtN1 << " neg." << endl << endl;
    cout << "Total number of Tot  events: " << iPWt1 + iPWtN1 << " pos. " << iNWt1 + iNWtN1 << " neg." << endl;
    cout << "Total number of Born events: " << iBornPWt1 + iBornPWtN1 << " pos. " << iBornNWt1 + iBornNWtN1 << " neg." << endl << endl;
    cout << "================================================================" << endl;
    cout << "Number of Sobr events with Wt > 1: " << iSobrPWtN1 << " pos. " << iSobrNWtN1 << " neg." << endl;
    cout << "Number of Virt events with Wt > 1: " << iVirtPWtN1 << " pos. " << iVirtNWtN1 << " neg." << endl;
    cout << "Number of Brdq events with Wt > 1: " << iBrdqPWtN1 << " pos. " << iBrdqNWtN1 << " neg." << endl;
    cout << "Number of Hard events with Wt > 1: " << iHardPWtN1 << " pos. " << iHardNWtN1 << " neg." << endl;
    cout << "Number of Ibbr events with Wt > 1: " << iIbbrPWtN1 << " pos. " << iIbbrNWtN1 << " neg." << endl;
    cout << "Number of Ibs1 events with Wt > 1: " << iIbs1PWtN1 << " pos. " << iIbs1NWtN1 << " neg." << endl;
    cout << "Number of Ibs2 events with Wt > 1: " << iIbs2PWtN1 << " pos. " << iIbs2NWtN1 << " neg." << endl << endl;
    cout << "Number of Tot  events with Wt > 1: " << iPWtN1 << " pos. " << iNWtN1 << " neg." << endl;
    cout << "Number of Born events with Wt > 1: " << iBornPWtN1 << " pos. " << iBornNWtN1 << " neg." << endl << endl;
    cout << "================================================================" << endl;
    cout << "Relative number of Sobr events with Wt > 1: " << (double) (iSobrPWtN1+iSobrNWtN1)/(iSobrPWt1+iSobrNWt1+iSobrPWtN1+iSobrNWtN1) << endl;
    cout << "Relative number of Virt events with Wt > 1: " << (double) (iVirtPWtN1+iVirtNWtN1)/(iVirtPWt1+iVirtNWt1+iVirtPWtN1+iVirtNWtN1) << endl;
    cout << "Relative number of Brdq events with Wt > 1: " << (double) (iBrdqPWtN1+iBrdqNWtN1)/(iBrdqPWt1+iBrdqNWt1+iBrdqPWtN1+iBrdqNWtN1) << endl;
    cout << "Relative number of Hard events with Wt > 1: " << (double) (iHardPWtN1+iHardNWtN1)/(iHardPWt1+iHardNWt1+iHardPWtN1+iHardNWtN1) << endl;
    cout << "Relative number of Ibbr events with Wt > 1: " << (double) (iIbbrPWtN1+iIbbrNWtN1)/(iIbbrPWt1+iIbbrNWt1+iIbbrPWtN1+iIbbrNWtN1) << endl;
    cout << "Relative number of Ibs1 events with Wt > 1: " << (double) (iIbs1PWtN1+iIbs1NWtN1)/(iIbs1PWt1+iIbs1NWt1+iIbs1PWtN1+iIbs1NWtN1) << endl;
    cout << "Relative number of Ibs2 events with Wt > 1: " << (double) (iIbs2PWtN1+iIbs2NWtN1)/(iIbs2PWt1+iIbs2NWt1+iIbs2PWtN1+iIbs2NWtN1) << endl;
    cout << "Relative number of Tot  events with Wt > 1: " << (double) (iPWtN1+iNWtN1)/(iPWt1+iNWt1+iPWtN1+iNWtN1) << endl;
    cout << "Relative number of Born events with Wt > 1: " << (double) (iBornPWtN1+iBornNWtN1)/(iBornPWt1+iBornNWt1+iBornPWtN1+iBornNWtN1) << endl << endl;
    cout << "***** End of Program *****" << endl;
//--------------------------------------------------------------------------------------------------
}
//-- end of main program ---------------------------------------------------------------------------
