************************************************************************
* File (nc_ha_1414_1616.f) created on Sun Nov  4 10:37:00 CET 2007.
************************************************************************
* This is the FORTRAN module (nc_ha_1414_1616) to calculate differential
* hard photon Bremsstrahlung for dn + dn -> mo + mo process.
*
* It is produced by s2n package of SANC (v1.10) Project.
* Copyright (C), SANC Project Team, 2002-2007.
*
* http://sanc.jinr.ru/
* http://pcphsanc.cern.ch/
* E-mail: <sanc@jinr.ru>
************************************************************************
      subroutine nc_ha_1414_1616 (s,spr,ph4,costh4,costh5,hard)
      implicit none!
      include 's2n_declare.h'

      real*8 kappaz,sq,sqspr
      real*8 a1z,a2z,v1z,v2z,vaz
      real*8 p1p2,p1p3,p1p4,p1p5,p2p3,p2p4,p2p5,p3p4,p3p5,p4p5
      real*8 ph4,cosph4,sinph4,costh4,costh5,sinth4,sinth5
      complex*16 chizs,chizsc,chizspr,chizsprc

      real*8 betasdndn,ibetasdndn,betasprmomo,ibetasprmomo
      real*8 z1dn,z2dn,z3mo,z4mo,iz1dn,iz2dn,iz3mo,iz4mo
      real*8 a0s,a0spr,v0s,v0spr,a2sspr,v2sspr,vas,vasspr,vaspr,a1sspr

      sq = dsqrt(s)
      sqspr = dsqrt(spr)

      cosph4 = dcos(ph4)
      sinph4 = dsin(ph4)
      sinth4 = dsqrt(1d0-costh4**2)
      sinth5 = dsqrt(1d0-costh5**2)

      kappaz   = gf*rzm2/(dsqrt(2d0)*8d0*pi)/alpha
      chizs    = 4d0*kappaz*s/(s-mz2)
      chizsc   = dconjg(chizs)
      chizspr  = 4d0*kappaz*spr/(spr-mz2)
      chizsprc = dconjg(chizspr)

      a1z = adn*amo
      a2z = 4d0*vdn*adn*vmo*amo
      v1z = vdn*vmo
      v2z = (vdn**2+adn**2)*(vmo**2+amo**2)
      vaz = (vdn**2+adn**2)*(vmo**2-amo**2)

      betasdndn = sqrt(1d0-4d0*rdnm2/s)
      ibetasdndn = 1d0/betasdndn
      betasprmomo = sqrt(1d0-4d0*rmom2/spr)
      ibetasprmomo = 1d0/betasprmomo
      
      a0s =
     & +qdn*qmo*(chizs+chizsc)*a1z
     & +chizs*chizsc*a2z
      
      a0spr =
     & +qdn*qmo*(chizspr+chizsprc)*a1z
     & +chizspr*chizsprc*a2z
      
      v0s =
     & +qdn**2*qmo**2
     & +qdn*qmo*(chizs+chizsc)*v1z
     & +chizs*chizsc*v2z
      
      v0spr =
     & +qdn**2*qmo**2
     & +qdn*qmo*(chizspr+chizsprc)*v1z
     & +chizspr*chizsprc*v2z
      
      a2sspr =
     & +qdn*qmo*(chizs+chizsc+chizspr+chizsprc)*a1z
     & +(chizs*chizsprc+chizsc*chizspr)*a2z
      
      v2sspr =
     & +2d0*qdn**2*qmo**2
     & +qdn*qmo*(chizs+chizsc+chizspr+chizsprc)*v1z
     & +(chizs*chizsprc+chizsc*chizspr)*v2z
      
      vas =
     & +qdn**2*qmo**2
     & +qdn*qmo*(chizs+chizsc)*v1z
     & +chizs*chizsc*vaz
      
      vasspr =
     & +2d0*qdn**2*qmo**2
     & +qdn*qmo*(chizs+chizsc+chizspr+chizsprc)*v1z
     & +(chizs*chizsprc+chizsc*chizspr)*vaz
      
      vaspr =
     & +qdn**2*qmo**2
     & +qdn*qmo*(chizspr+chizsprc)*v1z
     & +chizspr*chizsprc*vaz
      
      a1sspr =
     & -(chizs+chizsc-chizspr-chizsprc)*a1z

      z1dn=
     & +betasdndn*costh5*(1d0/2*s-1d0/2*spr)
     & +1d0/2*s-1d0/2*spr
      iz1dn = 1d0/z1dn

      z2dn=
     & +betasdndn*costh5*(-1d0/2*s+1d0/2*spr)
     & +1d0/2*s-1d0/2*spr
      iz2dn = 1d0/z2dn

      z3mo=
     & +betasprmomo*costh4*(1d0/2*s-1d0/2*spr)
     & +1d0/2*s-1d0/2*spr
      iz3mo = 1d0/z3mo

      z4mo=
     & +betasprmomo*costh4*(-1d0/2*s+1d0/2*spr)
     & +1d0/2*s-1d0/2*spr
      iz4mo = 1d0/z4mo

      p1p4=
     & +betasprmomo*sinph4*sinth4*sinth5*(1d0/4*sq*sqspr)
     & +betasprmomo*costh4*costh5*(-1d0/8*s-1d0/8*spr)
     & +betasprmomo*costh4*(1d0/8*s-1d0/8*spr)
     & +costh5*(1d0/8*s-1d0/8*spr)
     & -1d0/8*s-1d0/8*spr

      p2p4=
     & +betasprmomo*sinph4*sinth4*sinth5*(-1d0/4*sq*sqspr)
     & +betasprmomo*costh4*costh5*(1d0/8*s+1d0/8*spr)
     & +betasprmomo*costh4*(1d0/8*s-1d0/8*spr)
     & +costh5*(-1d0/8*s+1d0/8*spr)
     & -1d0/8*s-1d0/8*spr

      p1p3=
     & +betasprmomo*sinph4*sinth4*sinth5*(-1d0/4*sq*sqspr)
     & +betasprmomo*costh4*costh5*(1d0/8*s+1d0/8*spr)
     & +betasprmomo*costh4*(-1d0/8*s+1d0/8*spr)
     & +costh5*(1d0/8*s-1d0/8*spr)
     & -1d0/8*s-1d0/8*spr

      p2p3=
     & +betasprmomo*sinph4*sinth4*sinth5*(1d0/4*sq*sqspr)
     & +betasprmomo*costh4*costh5*(-1d0/8*s-1d0/8*spr)
     & +betasprmomo*costh4*(-1d0/8*s+1d0/8*spr)
     & +costh5*(-1d0/8*s+1d0/8*spr)
     & -1d0/8*s-1d0/8*spr

      p1p5=
     & +costh5*(-1d0/4*s+1d0/4*spr)
     & -1d0/4*s+1d0/4*spr

      p2p5=
     & +costh5*(1d0/4*s-1d0/4*spr)
     & -1d0/4*s+1d0/4*spr

      p3p5=
     & +betasprmomo*costh4*(-1d0/4*s+1d0/4*spr)
     & -1d0/4*s+1d0/4*spr

      p4p5=
     & +betasprmomo*costh4*(1d0/4*s-1d0/4*spr)
     & -1d0/4*s+1d0/4*spr

      hard=
     & +a0s*iz3mo**2*betasprmomo/pi*alpha**3*qmo**2*mmo**2*(s-spr)*(-1d0
     & /3*p1p3*p2p4/s**4+1d0/3*p1p4*p2p3/s**4+1d0/3*p1p4*p2p5/s**4-1d0/3
     & *p1p5*p2p4/s**4)
     & +a0s*iz3mo*iz4mo*betasprmomo/pi*alpha**3*qmo**2*mmo**2*(s-spr)*(-
     & 2d0/3*p1p3*p2p4/s**4-1d0/3*p1p3*p2p5/s**4+2d0/3*p1p4*p2p3/s**4+1d
     & 0/3*p1p4*p2p5/s**4+1d0/3*p1p5*p2p3/s**4-1d0/3*p1p5*p2p4/s**4)
     & +a0s*iz3mo*iz4mo*betasprmomo/pi*alpha**3*qmo**2*(s-spr)*(1d0/3*p1
     & p3*p2p4/s**4*spr+1d0/6*p1p3*p2p5/s**4*spr-1d0/3*p1p4*p2p3/s**4*sp
     & r-1d0/6*p1p4*p2p5/s**4*spr-1d0/6*p1p5*p2p3/s**4*spr+1d0/6*p1p5*p2
     & p4/s**4*spr)
     & +a0s*iz3mo*betasprmomo/pi*alpha**3*qmo**2*(s-spr)*(1d0/6*p1p3*p2p
     & 4/s**4-1d0/6*p1p4*p2p3/s**4-1d0/6*p1p4*p2p5/s**4+1d0/6*p1p5*p2p4/
     & s**4)
     & +a0s*iz4mo**2*betasprmomo/pi*alpha**3*qmo**2*mmo**2*(s-spr)*(-1d0
     & /3*p1p3*p2p4/s**4-1d0/3*p1p3*p2p5/s**4+1d0/3*p1p4*p2p3/s**4+1d0/3
     & *p1p5*p2p3/s**4)
     & +a0s*iz4mo*betasprmomo/pi*alpha**3*qmo**2*(s-spr)*(1d0/6*p1p3*p2p
     & 4/s**4+1d0/6*p1p3*p2p5/s**4-1d0/6*p1p4*p2p3/s**4-1d0/6*p1p5*p2p3/
     & s**4)
     & +a0spr*iz1dn**2*betasprmomo/pi*alpha**3*qdn**2*(s-spr)*(-1d0/3*p1
     & p3*p2p4*mdn**2/s**2/spr**2+1d0/3*p1p4*p2p3*mdn**2/s**2/spr**2-1d0
     & /3*p2p3*p4p5*mdn**2/s**2/spr**2+1d0/3*p2p4*p3p5*mdn**2/s**2/spr**
     & 2)
     & +a0spr*iz1dn*iz2dn*betasprmomo/pi*alpha**3*qdn**2*(s-spr)*(1d0/3*
     & p1p3*p2p4/s/spr**2-1d0/6*p1p3*p4p5/s/spr**2-1d0/3*p1p4*p2p3/s/spr
     & **2+1d0/6*p1p4*p3p5/s/spr**2+1d0/6*p2p3*p4p5/s/spr**2-1d0/6*p2p4*
     & p3p5/s/spr**2)
     & +a0spr*iz1dn*betasprmomo/pi*alpha**3*qdn**2*(s-spr)*(-1d0/6*p1p3*
     & p2p4/s**2/spr**2+1d0/6*p1p4*p2p3/s**2/spr**2-1d0/6*p2p3*p4p5/s**2
     & /spr**2+1d0/6*p2p4*p3p5/s**2/spr**2)
     & +a0spr*iz2dn**2*betasprmomo/pi*alpha**3*qdn**2*(s-spr)*(-1d0/3*p1
     & p3*p2p4*mdn**2/s**2/spr**2+1d0/3*p1p3*p4p5*mdn**2/s**2/spr**2+1d0
     & /3*p1p4*p2p3*mdn**2/s**2/spr**2-1d0/3*p1p4*p3p5*mdn**2/s**2/spr**
     & 2)
     & +a0spr*iz2dn*betasprmomo/pi*alpha**3*qdn**2*(s-spr)*(-1d0/6*p1p3*
     & p2p4/s**2/spr**2+1d0/6*p1p3*p4p5/s**2/spr**2+1d0/6*p1p4*p2p3/s**2
     & /spr**2-1d0/6*p1p4*p3p5/s**2/spr**2)
     & +v0s*iz3mo**2*betasprmomo/pi*alpha**3*qmo**2*mmo**2*(s-spr)*(-1d0
     & /3*p1p3*p2p4/s**4-1d0/3*p1p4*p2p3/s**4-1d0/3*p1p4*p2p5/s**4-1d0/3
     & *p1p5*p2p4/s**4)
     & +v0s*iz3mo*iz4mo*betasprmomo/pi*alpha**3*qmo**2*mmo**2*(s-spr)*(-
     & 2d0/3*p1p3*p2p4/s**4-1d0/3*p1p3*p2p5/s**4-2d0/3*p1p4*p2p3/s**4-1d
     & 0/3*p1p4*p2p5/s**4-1d0/3*p1p5*p2p3/s**4-1d0/3*p1p5*p2p4/s**4)
     & +v0s*iz3mo*iz4mo*betasprmomo/pi*alpha**3*qmo**2*(s-spr)*(1d0/3*p1
     & p3*p2p4/s**4*spr+1d0/6*p1p3*p2p5/s**4*spr+1d0/3*p1p4*p2p3/s**4*sp
     & r+1d0/6*p1p4*p2p5/s**4*spr+1d0/6*p1p5*p2p3/s**4*spr+1d0/6*p1p5*p2
     & p4/s**4*spr)
     & +v0s*iz3mo*betasprmomo/pi*alpha**3*qmo**2*(s-spr)*(-1d0/3*p1p3*p2
     & p3/s**4+1d0/6*p1p3*p2p4/s**4+1d0/6*p1p4*p2p3/s**4+1d0/6*p1p4*p2p5
     & /s**4+1d0/6*p1p5*p2p4/s**4)
     & +v0s*iz4mo**2*betasprmomo/pi*alpha**3*qmo**2*mmo**2*(s-spr)*(-1d0
     & /3*p1p3*p2p4/s**4-1d0/3*p1p3*p2p5/s**4-1d0/3*p1p4*p2p3/s**4-1d0/3
     & *p1p5*p2p3/s**4)
     & +v0s*iz4mo*betasprmomo/pi*alpha**3*qmo**2*(s-spr)*(1d0/6*p1p3*p2p
     & 4/s**4+1d0/6*p1p3*p2p5/s**4+1d0/6*p1p4*p2p3/s**4-1d0/3*p1p4*p2p4/
     & s**4+1d0/6*p1p5*p2p3/s**4)
     & +v0spr*iz1dn**2*betasprmomo/pi*alpha**3*qdn**2*(s-spr)*(-1d0/3*p1
     & p3*p2p4*mdn**2/s**2/spr**2-1d0/3*p1p4*p2p3*mdn**2/s**2/spr**2+1d0
     & /3*p2p3*p4p5*mdn**2/s**2/spr**2+1d0/3*p2p4*p3p5*mdn**2/s**2/spr**
     & 2)
     & +v0spr*iz1dn*iz2dn*betasprmomo/pi*alpha**3*qdn**2*(s-spr)*(1d0/3*
     & p1p3*p2p4/s/spr**2-1d0/6*p1p3*p4p5/s/spr**2+1d0/3*p1p4*p2p3/s/spr
     & **2-1d0/6*p1p4*p3p5/s/spr**2-1d0/6*p2p3*p4p5/s/spr**2-1d0/6*p2p4*
     & p3p5/s/spr**2)
     & +v0spr*iz1dn*betasprmomo/pi*alpha**3*qdn**2*(s-spr)*(1d0/3*p1p3*p
     & 1p4/s**2/spr**2-1d0/6*p1p3*p2p4/s**2/spr**2-1d0/6*p1p4*p2p3/s**2/
     & spr**2+1d0/6*p2p3*p4p5/s**2/spr**2+1d0/6*p2p4*p3p5/s**2/spr**2)
     & +v0spr*iz2dn**2*betasprmomo/pi*alpha**3*qdn**2*(s-spr)*(-1d0/3*p1
     & p3*p2p4*mdn**2/s**2/spr**2+1d0/3*p1p3*p4p5*mdn**2/s**2/spr**2-1d0
     & /3*p1p4*p2p3*mdn**2/s**2/spr**2+1d0/3*p1p4*p3p5*mdn**2/s**2/spr**
     & 2)
     & +v0spr*iz2dn*betasprmomo/pi*alpha**3*qdn**2*(s-spr)*(-1d0/6*p1p3*
     & p2p4/s**2/spr**2+1d0/6*p1p3*p4p5/s**2/spr**2-1d0/6*p1p4*p2p3/s**2
     & /spr**2+1d0/6*p1p4*p3p5/s**2/spr**2+1d0/3*p2p3*p2p4/s**2/spr**2)
     & +a2sspr*iz1dn*iz3mo*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)*(-1d0
     & /3*p1p3*p1p4*p2p3/s**3/spr+1d0/3*p1p3**2*p2p4/s**3/spr)
     & +a2sspr*iz1dn*iz3mo*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)**2*(1
     & d0/12*p1p3*p1p4/s**3/spr-1d0/12*p1p3*p2p3/s**3/spr)
     & +a2sspr*iz1dn*iz4mo*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)*(-1d0
     & /3*p1p3*p1p4*p2p4/s**3/spr+1d0/3*p1p4**2*p2p3/s**3/spr)
     & +a2sspr*iz1dn*iz4mo*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)**2*(1
     & d0/12*p1p3*p1p4/s**3/spr-1d0/12*p1p4*p2p4/s**3/spr)
     & +a2sspr*iz1dn*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)*(1d0/12*p1p
     & 3*p2p3/s**3/spr+1d0/12*p1p3*p2p4/s**3/spr+1d0/24*p1p3/s**2/spr+1d
     & 0/12*p1p4*p2p3/s**3/spr+1d0/12*p1p4*p2p4/s**3/spr+1d0/24*p1p4/s**
     & 2/spr)
     & +a2sspr*iz2dn*iz3mo*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)*(-1d0
     & /3*p1p3*p2p3*p2p4/s**3/spr+1d0/3*p1p4*p2p3**2/s**3/spr)
     & +a2sspr*iz2dn*iz3mo*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)**2*(-
     & 1d0/12*p1p3*p2p3/s**3/spr+1d0/12*p2p3*p2p4/s**3/spr)
     & +a2sspr*iz2dn*iz4mo*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)*(1d0/
     & 3*p1p3*p2p4**2/s**3/spr-1d0/3*p1p4*p2p3*p2p4/s**3/spr)
     & +a2sspr*iz2dn*iz4mo*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)**2*(-
     & 1d0/12*p1p4*p2p4/s**3/spr+1d0/12*p2p3*p2p4/s**3/spr)
     & +a2sspr*iz2dn*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)*(1d0/12*p1p
     & 3*p2p3/s**3/spr+1d0/12*p1p3*p2p4/s**3/spr+1d0/12*p1p4*p2p3/s**3/s
     & pr+1d0/12*p1p4*p2p4/s**3/spr+1d0/24*p2p3/s**2/spr+1d0/24*p2p4/s**
     & 2/spr)
     & +a2sspr*iz3mo*betasprmomo/pi*alpha**3*qdn*qmo*mmo**2*(s-spr)*(1d0
     & /12*p1p3/s**3/spr+1d0/12*p1p4/s**3/spr+1d0/12*p2p3/s**3/spr+1d0/1
     & 2*p2p4/s**3/spr)
     & +a2sspr*iz3mo*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)*(-1d0/12*p1
     & p3*p1p4/s**3/spr-1d0/12*p1p3*p2p4/s**3/spr-1d0/24*p1p3/s**3-1d0/1
     & 2*p1p4*p2p3/s**3/spr-1d0/12*p2p3*p2p4/s**3/spr-1d0/24*p2p3/s**3)
     & +a2sspr*iz4mo*betasprmomo/pi*alpha**3*qdn*qmo*mmo**2*(s-spr)*(1d0
     & /12*p1p3/s**3/spr+1d0/12*p1p4/s**3/spr+1d0/12*p2p3/s**3/spr+1d0/1
     & 2*p2p4/s**3/spr)
     & +a2sspr*iz4mo*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)*(-1d0/12*p1
     & p3*p1p4/s**3/spr-1d0/12*p1p3*p2p4/s**3/spr-1d0/12*p1p4*p2p3/s**3/
     & spr-1d0/24*p1p4/s**3-1d0/12*p2p3*p2p4/s**3/spr-1d0/24*p2p4/s**3)
     & +a2sspr*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)*(-1d0/12*p1p3/s**
     & 3/spr-1d0/12*p1p4/s**3/spr-1d0/12*p2p3/s**3/spr-1d0/12*p2p4/s**3/
     & spr)
     & +v2sspr*iz1dn*iz3mo*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)*(1d0/
     & 3*p1p3*p1p4*p2p3/s**3/spr+1d0/3*p1p3**2*p2p4/s**3/spr)
     & +v2sspr*iz1dn*iz3mo*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)**2*(-
     & 1d0/12*p1p3*p1p4/s**3/spr+1d0/12*p1p3*p2p3/s**3/spr)
     & +v2sspr*iz1dn*iz4mo*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)*(-1d0
     & /3*p1p3*p1p4*p2p4/s**3/spr-1d0/3*p1p4**2*p2p3/s**3/spr)
     & +v2sspr*iz1dn*iz4mo*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)**2*(1
     & d0/12*p1p3*p1p4/s**3/spr-1d0/12*p1p4*p2p4/s**3/spr)
     & +v2sspr*iz1dn*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)*(-1d0/12*p1
     & p3*p2p3/s**3/spr+1d0/12*p1p3*p2p4/s**3/spr+1d0/24*p1p3/s**2/spr-1
     & d0/12*p1p4*p2p3/s**3/spr+1d0/12*p1p4*p2p4/s**3/spr-1d0/24*p1p4/s*
     & *2/spr)
     & +v2sspr*iz2dn*iz3mo*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)*(-1d0
     & /3*p1p3*p2p3*p2p4/s**3/spr-1d0/3*p1p4*p2p3**2/s**3/spr)
     & +v2sspr*iz2dn*iz3mo*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)**2*(-
     & 1d0/12*p1p3*p2p3/s**3/spr+1d0/12*p2p3*p2p4/s**3/spr)
     & +v2sspr*iz2dn*iz4mo*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)*(1d0/
     & 3*p1p3*p2p4**2/s**3/spr+1d0/3*p1p4*p2p3*p2p4/s**3/spr)
     & +v2sspr*iz2dn*iz4mo*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)**2*(1
     & d0/12*p1p4*p2p4/s**3/spr-1d0/12*p2p3*p2p4/s**3/spr)
     & +v2sspr*iz2dn*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)*(1d0/12*p1p
     & 3*p2p3/s**3/spr+1d0/12*p1p3*p2p4/s**3/spr-1d0/12*p1p4*p2p3/s**3/s
     & pr-1d0/12*p1p4*p2p4/s**3/spr-1d0/24*p2p3/s**2/spr+1d0/24*p2p4/s**
     & 2/spr)
     & +v2sspr*iz3mo*betasprmomo/pi*alpha**3*qdn*qmo*mmo**2*(s-spr)*(1d0
     & /12*p1p3/s**3/spr-1d0/12*p1p4/s**3/spr-1d0/12*p2p3/s**3/spr+1d0/1
     & 2*p2p4/s**3/spr)
     & +v2sspr*iz3mo*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)*(1d0/12*p1p
     & 3*p1p4/s**3/spr-1d0/12*p1p3*p2p4/s**3/spr-1d0/24*p1p3/s**3+1d0/12
     & *p1p4*p2p3/s**3/spr-1d0/12*p2p3*p2p4/s**3/spr+1d0/24*p2p3/s**3)
     & +v2sspr*iz4mo*betasprmomo/pi*alpha**3*qdn*qmo*mmo**2*(s-spr)*(1d0
     & /12*p1p3/s**3/spr-1d0/12*p1p4/s**3/spr-1d0/12*p2p3/s**3/spr+1d0/1
     & 2*p2p4/s**3/spr)
     & +v2sspr*iz4mo*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)*(-1d0/12*p1
     & p3*p1p4/s**3/spr-1d0/12*p1p3*p2p4/s**3/spr+1d0/12*p1p4*p2p3/s**3/
     & spr+1d0/24*p1p4/s**3+1d0/12*p2p3*p2p4/s**3/spr-1d0/24*p2p4/s**3)
     & +v2sspr*betasprmomo/pi*alpha**3*qdn*qmo*(s-spr)*(-1d0/12*p1p3/s**
     & 3/spr+1d0/12*p1p4/s**3/spr+1d0/12*p2p3/s**3/spr-1d0/12*p2p4/s**3/
     & spr)
     & +vas*iz3mo**2*betasprmomo/pi*alpha**3*qmo**2*mmo**4*(s-spr)*(-1d0
     & /6/s**3)
     & +vas*iz3mo*iz4mo*betasprmomo/pi*alpha**3*qmo**2*mmo**2*(s-spr)*(1
     & d0/6/s**3*spr-2d0/3*p1p5*p2p5/s**4)
     & +vas*iz3mo*iz4mo*betasprmomo/pi*alpha**3*qmo**2*mmo**4*(s-spr)*(-
     & 1d0/3/s**3)
     & +vas*iz4mo**2*betasprmomo/pi*alpha**3*qmo**2*mmo**4*(s-spr)*(-1d0
     & /6/s**3)
     & +vasspr*iz1dn*iz3mo*betasprmomo*(s+spr)/pi*alpha**3*qdn*qmo*mmo**
     & 2*(s-spr)*(1d0/12*p1p3/s**3/spr)
     & +vasspr*iz1dn*iz4mo*betasprmomo*(s+spr)/pi*alpha**3*qdn*qmo*mmo**
     & 2*(s-spr)*(-1d0/12*p1p4/s**3/spr)
     & +vasspr*iz2dn*iz3mo*betasprmomo*(s+spr)/pi*alpha**3*qdn*qmo*mmo**
     & 2*(s-spr)*(-1d0/12*p2p3/s**3/spr)
     & +vasspr*iz2dn*iz4mo*betasprmomo*(s+spr)/pi*alpha**3*qdn*qmo*mmo**
     & 2*(s-spr)*(1d0/12*p2p4/s**3/spr)
     & +vasspr*iz3mo*betasprmomo/pi*alpha**3*qdn*qmo*mmo**2*(s-spr)*(1d0
     & /12*p1p5/s**3/spr-1d0/12*p2p5/s**3/spr)
     & +vasspr*iz4mo*betasprmomo/pi*alpha**3*qdn*qmo*mmo**2*(s-spr)*(-1d
     & 0/12*p1p5/s**3/spr+1d0/12*p2p5/s**3/spr)
     & +vaspr*iz1dn**2*betasprmomo/pi*alpha**3*qdn**2*mmo**2*(s-spr)*(-1
     & d0/6*mdn**2/s**2/spr)
     & +vaspr*iz1dn*iz2dn*betasprmomo/pi*alpha**3*qdn**2*mmo**2*(s-spr)*
     & (1d0/6/spr**2)
     & +vaspr*iz1dn*betasprmomo/pi*alpha**3*qdn**2*mmo**2*(s-spr)*(-1d0/
     & 6*mdn**2/s**2/spr**2-1d0/12/s**2/spr-1d0/12/s/spr**2)
     & +vaspr*iz2dn**2*betasprmomo/pi*alpha**3*qdn**2*mmo**2*(s-spr)*(-1
     & d0/6*mdn**2/s**2/spr)
     & +vaspr*iz2dn*betasprmomo/pi*alpha**3*qdn**2*mmo**2*(s-spr)*(-1d0/
     & 6*mdn**2/s**2/spr**2-1d0/12/s**2/spr-1d0/12/s/spr**2)
     & +vaspr*betasprmomo/pi*alpha**3*qdn**2*mmo**2*(s-spr)*(-1d0/6/s**2
     & /spr**2)
     & +a1sspr*iz3mo*betasprmomo*(s+spr)/pi*alpha**3*qdn**2*qmo**2*mmo**
     & 2*(s-spr)*(1d0/24/s**3/spr)
     & +a1sspr*iz4mo*betasprmomo*(s+spr)/pi*alpha**3*qdn**2*qmo**2*mmo**
     & 2*(s-spr)*(1d0/24/s**3/spr)

      hard = hard*conhc

      return
      end
